import unittest

from Tests.utils.utils import get_test_path
from kalliope.core.Models import Brain
from kalliope.core.Models import Neuron
from kalliope.core.Models import Synapse
from kalliope.core.Models.MatchedSynapse import MatchedSynapse
from kalliope.core.Models.Signal import Signal
from kalliope.core.OrderAnalyser import OrderAnalyser


class TestOrderAnalyser(unittest.TestCase):
    """Test case for the OrderAnalyser Class"""

    def setUp(self):
        self.correction_file_to_test = get_test_path("files/test-stt-correction.yml")

    def test_get_matching_synapse(self):
        # Init
        neuron1 = Neuron(name='neurone1', parameters={'var1': 'val1'})
        neuron2 = Neuron(name='neurone2', parameters={'var2': 'val2'})
        neuron3 = Neuron(name='neurone3', parameters={'var3': 'val3'})
        neuron4 = Neuron(name='neurone4', parameters={'var4': 'val4'})

        signal1 = Signal(name="order", parameters="this is the sentence")
        signal2 = Signal(name="order", parameters="this is the second sentence")
        signal3 = Signal(name="order", parameters="that is part of the third sentence")
        signal4 = Signal(name="order", parameters={"matching-type": "strict",
                                                   "text": "that is part of the fourth sentence"})
        signal5 = Signal(name="order", parameters={"matching-type": "ordered-strict",
                                                   "text": "sentence 5 with specific order"})
        signal6 = Signal(name="order", parameters={"matching-type": "normal",
                                                   "text": "matching type normal"})
        signal7 = Signal(name="order", parameters={"matching-type": "non-existing",
                                                   "text": "matching type non existing"})
        signal8 = Signal(name="order", parameters={"matching-type": "non-existing",
                                                   "non-existing-parameter": "will not match order"})
        signal9 = Signal(name="order", parameters="order that should be triggered because synapse is disabled")
        signal10 = Signal(name="order", parameters="i say this")
        signal11 = Signal(name="order", parameters="and then that")
        signal12 = Signal(name="order", parameters={"matching-type": "strict",
                                                    "text": "just 1 test",
                                                    "stt-correction": [
                                                        {"input": "one", "output": "1"}
                                                    ]})
        signal13 = Signal(name="order", parameters={"matching-type": "not-contain",
                                                    "text": "testing the not contain",
                                                    "excluded-words": ["that", "in"]})

        synapse1 = Synapse(name="Synapse1", neurons=[neuron1, neuron2], signals=[signal1])
        synapse2 = Synapse(name="Synapse2", neurons=[neuron3, neuron4], signals=[signal2])
        synapse3 = Synapse(name="Synapse3", neurons=[neuron2, neuron4], signals=[signal3])
        synapse4 = Synapse(name="Synapse4", neurons=[neuron2, neuron4], signals=[signal4])
        synapse5 = Synapse(name="Synapse5", neurons=[neuron1, neuron2], signals=[signal5])
        synapse6 = Synapse(name="Synapse6", neurons=[neuron1, neuron2], signals=[signal6])
        synapse7 = Synapse(name="Synapse7", neurons=[neuron1, neuron2], signals=[signal7])
        synapse8 = Synapse(name="Synapse8", neurons=[neuron1, neuron2], signals=[signal8])
        synapse9 = Synapse(name="Synapse9", enabled=False, neurons=[neuron1, neuron2], signals=[signal9])
        synapse10 = Synapse(name="Synapse10", neurons=[neuron1], signals=[signal10, signal11])
        synapse11 = Synapse(name="Synapse11", neurons=[neuron1], signals=[signal12])
        synapse12 = Synapse(name="Synapse13", neurons=[neuron1], signals=[signal13])

        all_synapse_list = [synapse1,
                            synapse2,
                            synapse3,
                            synapse4,
                            synapse5,
                            synapse6,
                            synapse7,
                            synapse8,
                            synapse9,
                            synapse10,
                            synapse11,
                            synapse12]

        br = Brain(synapses=all_synapse_list)

        # TEST1: should return synapse1
        spoken_order = "this is the sentence"

        # Create the matched synapse
        expected_matched_synapse_1 = MatchedSynapse(matched_synapse=synapse1,
                                                    matched_order=spoken_order,
                                                    user_order=spoken_order)

        matched_synapses = OrderAnalyser.get_matching_synapse(order=spoken_order, brain=br)
        self.assertEqual(len(matched_synapses), 1)
        self.assertTrue(expected_matched_synapse_1 in matched_synapses)

        # with defined normal matching type
        spoken_order = "matching type normal"
        expected_matched_synapse_5 = MatchedSynapse(matched_synapse=synapse6,
                                                    matched_order=spoken_order,
                                                    user_order=spoken_order)

        matched_synapses = OrderAnalyser.get_matching_synapse(order=spoken_order, brain=br)
        self.assertEqual(len(matched_synapses), 1)
        self.assertTrue(expected_matched_synapse_5 in matched_synapses)

        # TEST2: should return synapse1 and 2
        spoken_order = "this is the second sentence"
        expected_matched_synapse_2 = MatchedSynapse(matched_synapse=synapse1,
                                                    matched_order=spoken_order,
                                                    user_order=spoken_order)
        matched_synapses = OrderAnalyser.get_matching_synapse(order=spoken_order, brain=br)
        self.assertEqual(len(matched_synapses), 2)
        self.assertTrue(expected_matched_synapse_1, expected_matched_synapse_2 in matched_synapses)

        # TEST3: should empty
        spoken_order = "not a valid order"
        matched_synapses = OrderAnalyser.get_matching_synapse(order=spoken_order, brain=br)
        self.assertFalse(matched_synapses)

        # TEST4: with matching type strict
        spoken_order = "that is part of the fourth sentence"
        expected_matched_synapse_3 = MatchedSynapse(matched_synapse=synapse4,
                                                    matched_order=spoken_order,
                                                    user_order=spoken_order)
        matched_synapses = OrderAnalyser.get_matching_synapse(order=spoken_order, brain=br)
        self.assertTrue(expected_matched_synapse_3 in matched_synapses)

        spoken_order = "that is part of the fourth sentence with more word"
        matched_synapses = OrderAnalyser.get_matching_synapse(order=spoken_order, brain=br)
        self.assertFalse(matched_synapses)

        # TEST5: with matching type ordered strict
        spoken_order = "sentence 5 with specific order"
        expected_matched_synapse_4 = MatchedSynapse(matched_synapse=synapse5,
                                                    matched_order=spoken_order,
                                                    user_order=spoken_order)
        matched_synapses = OrderAnalyser.get_matching_synapse(order=spoken_order, brain=br)
        self.assertEqual(len(matched_synapses), 1)
        self.assertTrue(expected_matched_synapse_4 in matched_synapses)

        spoken_order = "order specific with 5 sentence"
        matched_synapses = OrderAnalyser.get_matching_synapse(order=spoken_order, brain=br)
        self.assertFalse(matched_synapses)

        # TEST6: non supported type of matching. should fallback to normal
        spoken_order = "matching type non existing"
        expected_matched_synapse_5 = MatchedSynapse(matched_synapse=synapse7,
                                                    matched_order=spoken_order,
                                                    user_order=spoken_order)
        matched_synapses = OrderAnalyser.get_matching_synapse(order=spoken_order, brain=br)
        self.assertTrue(expected_matched_synapse_5 in matched_synapses)

        # TEST7: should not match the disabled synapse
        spoken_order = "order that should be triggered because synapse is disabled"
        # we expect an empty list
        matched_synapses = OrderAnalyser.get_matching_synapse(order=spoken_order, brain=br)
        self.assertTrue(len(matched_synapses) == 0)

        # TEST8: a spoken order that match multiple time the same synapse should return the synapse only once
        spoken_order = "i say this and then that"
        matched_synapses = OrderAnalyser.get_matching_synapse(order=spoken_order, brain=br)
        self.assertTrue("Synapse10" in matched_synapse.synapse.name for matched_synapse in matched_synapses)
        self.assertTrue(len(matched_synapses) == 1)

        # TEST9: with STT correction
        spoken_order = "just one test"
        expected_order_with_correction = "just 1 test"
        matched_synapses = OrderAnalyser.get_matching_synapse(order=spoken_order, brain=br)
        self.assertTrue("Synapse11" in matched_synapse.synapse.name for matched_synapse in matched_synapses)
        self.assertEqual(expected_order_with_correction, next(matched_synapse.matched_order for matched_synapse in
                                                              matched_synapses))
        self.assertTrue(len(matched_synapses) == 1)

        # TEST10: with `not-contain`
        spoken_order = "testing the not contain"
        matched_synapses = OrderAnalyser.get_matching_synapse(order=spoken_order, brain=br)
        self.assertTrue("Synapse12" in matched_synapse.synapse.name for matched_synapse in matched_synapses)
        self.assertEqual(spoken_order,
                         next(matched_synapse.matched_order for matched_synapse in matched_synapses))
        self.assertTrue(len(matched_synapses) == 1)

        # test excluded word after the matching sentence
        spoken_order = "testing the not contain in "
        matched_synapses = OrderAnalyser.get_matching_synapse(order=spoken_order, brain=br)
        self.assertFalse(matched_synapses)
        self.assertTrue(len(matched_synapses) == 0)

        # test excluded word before the matching sequence
        spoken_order = "in testing the not contain"
        matched_synapses = OrderAnalyser.get_matching_synapse(order=spoken_order, brain=br)
        self.assertFalse(matched_synapses)
        self.assertTrue(len(matched_synapses) == 0)

        # test excluded word in the middle of the matching sequence
        spoken_order = "testing the in not contain"
        matched_synapses = OrderAnalyser.get_matching_synapse(order=spoken_order, brain=br)
        self.assertFalse(matched_synapses)
        self.assertTrue(len(matched_synapses) == 0)

    def test_get_split_order_without_bracket(self):
        # Success
        order_to_test = "this is the order"
        expected_result = ["this", "is", "the", "order"]
        self.assertEqual(OrderAnalyser._get_split_order_without_bracket(order_to_test), expected_result,
                         "No brackets Fails to return the expected list")

        order_to_test = "this is the {{ order }}"
        expected_result = ["this", "is", "the"]
        self.assertEqual(OrderAnalyser._get_split_order_without_bracket(order_to_test), expected_result,
                         "With spaced brackets Fails to return the expected list")

        order_to_test = "this is the {{order }}"  # left bracket without space
        expected_result = ["this", "is", "the"]
        self.assertEqual(OrderAnalyser._get_split_order_without_bracket(order_to_test), expected_result,
                         "Left brackets Fails to return the expected list")

        order_to_test = "this is the {{ order}}"  # right bracket without space
        expected_result = ["this", "is", "the"]
        self.assertEqual(OrderAnalyser._get_split_order_without_bracket(order_to_test), expected_result,
                         "Right brackets Fails to return the expected list")

        order_to_test = "this is the {{order}}"  # bracket without space
        expected_result = ["this", "is", "the"]
        self.assertEqual(OrderAnalyser._get_split_order_without_bracket(order_to_test), expected_result,
                         "No space brackets Fails to return the expected list")

    def test_is_normal_matching(self):
        # same order
        test_order = "expected order in the signal"
        test_signal = "expected order in the signal"

        self.assertTrue(OrderAnalyser.is_normal_matching(user_order=test_order,
                                                         signal_order=test_signal))

        # not the same order
        test_order = "this is an order"
        test_signal = "expected order in the signal"

        self.assertFalse(OrderAnalyser.is_normal_matching(user_order=test_order,
                                                          signal_order=test_signal))

        # same order with more word in the user order
        test_order = "expected order in the signal with more word"
        test_signal = "expected order in the signal"

        self.assertTrue(OrderAnalyser.is_normal_matching(user_order=test_order,
                                                         signal_order=test_signal))

        # same order with bracket
        test_order = "expected order in the signal"
        test_signal = "expected order in the signal {{ variable }}"

        self.assertTrue(OrderAnalyser.is_normal_matching(user_order=test_order,
                                                         signal_order=test_signal))

        # same order with bracket
        test_order = "expected order in the signal variable_to_catch"
        test_signal = "expected order in the signal {{ variable }}"

        self.assertTrue(OrderAnalyser.is_normal_matching(user_order=test_order,
                                                         signal_order=test_signal))

        # same order with bracket and words after brackets
        test_order = "expected order in the signal variable_to_catch other word"
        test_signal = "expected order in the signal {{ variable }} other word"

        self.assertTrue(OrderAnalyser.is_normal_matching(user_order=test_order,
                                                         signal_order=test_signal))

    def test_is_not_contain_matching(self):
        # Test the normal matching use case with no excluded words matching in the order
        test_order = "expected order in the signal"
        test_signal = "expected order in the signal"
        excluded_words = "test"

        self.assertTrue(OrderAnalyser.is_not_contain_matching(user_order=test_order,
                                                              signal_order=test_signal,
                                                              not_containing_words=excluded_words))

        # test with one excluded word matching in the order.
        test_order = "expected order in the signal"
        test_signal = "expected"
        excluded_words = ["order"]

        self.assertFalse(OrderAnalyser.is_not_contain_matching(user_order=test_order,
                                                               signal_order=test_signal,
                                                               not_containing_words=excluded_words))

        # test with multiple excluded word and only one matching in the order.
        test_order = "expected order in the signal"
        test_signal = "expected"
        excluded_words = ["blabla", "order", "bloblo"]

        self.assertFalse(OrderAnalyser.is_not_contain_matching(user_order=test_order,
                                                               signal_order=test_signal,
                                                               not_containing_words=excluded_words))

        # test with multiple excluded word and multiple are matching in the order.
        test_order = "expected order in the signal"
        test_signal = "expected"
        excluded_words = ["in", "order", "in"]

        self.assertFalse(OrderAnalyser.is_not_contain_matching(user_order=test_order,
                                                               signal_order=test_signal,
                                                               not_containing_words=excluded_words))

        # test with multiple excluded word and only none is matching the order
        test_order = "expected order in the signal"
        test_signal = "expected order in"
        excluded_words = ["blublu", "blabla", "bloblo"]

        self.assertTrue(OrderAnalyser.is_not_contain_matching(user_order=test_order,
                                                              signal_order=test_signal,
                                                              not_containing_words=excluded_words))

        # test with multiple excluded word and brackets.
        test_order = "expected order the signal"
        test_signal = "expected order {{ in }}"
        excluded_words = ["blublu", "blabla", "in"]

        self.assertTrue(OrderAnalyser.is_not_contain_matching(user_order=test_order,
                                                              signal_order=test_signal,
                                                              not_containing_words=excluded_words))

    def test_is_strict_matching(self):
        # same order with same amount of word
        test_order = "expected order in the signal"
        test_signal = "expected order in the signal"

        self.assertTrue(OrderAnalyser.is_strict_matching(user_order=test_order,
                                                         signal_order=test_signal))

        # same order but not the same amount of word
        test_order = "expected order in the signal with more word"
        test_signal = "expected order in the signal"

        self.assertFalse(OrderAnalyser.is_strict_matching(user_order=test_order,
                                                          signal_order=test_signal))

        # same order with same amount of word and brackets
        test_order = "expected order in the signal variable_to_catch"
        test_signal = "expected order in the signal {{ variable }}"

        self.assertTrue(OrderAnalyser.is_strict_matching(user_order=test_order,
                                                         signal_order=test_signal))

        # same order with same amount of word and brackets with words after last brackets
        test_order = "expected order in the signal variable_to_catch other word"
        test_signal = "expected order in the signal {{ variable }} other word"

        self.assertTrue(OrderAnalyser.is_strict_matching(user_order=test_order,
                                                         signal_order=test_signal))

        # same order with same amount of word and brackets with words after last brackets but more words
        test_order = "expected order in the signal variable_to_catch other word and more word"
        test_signal = "expected order in the signal {{ variable }} other word"

        self.assertFalse(OrderAnalyser.is_strict_matching(user_order=test_order,
                                                          signal_order=test_signal))

    def test_ordered_strict_matching(self):
        # same order with same amount of word with same order
        test_order = "expected order in the signal"
        test_signal = "expected order in the signal"
        self.assertTrue(OrderAnalyser.is_ordered_strict_matching(user_order=test_order,
                                                                 signal_order=test_signal))

        # same order with same amount of word without same order
        test_order = "signal the in order expected"
        test_signal = "expected order in the signal"
        self.assertFalse(OrderAnalyser.is_ordered_strict_matching(user_order=test_order,
                                                                  signal_order=test_signal))

        # same order with same amount of word and brackets in the same order
        test_order = "expected order in the signal variable_to_catch"
        test_signal = "expected order in the signal {{ variable }}"

        self.assertTrue(OrderAnalyser.is_ordered_strict_matching(user_order=test_order,
                                                                 signal_order=test_signal))

        # same order with same amount of word and brackets in the same order with words after bracket
        test_order = "expected order in the signal variable_to_catch with word"
        test_signal = "expected order in the signal {{ variable }} with word"

        self.assertTrue(OrderAnalyser.is_ordered_strict_matching(user_order=test_order,
                                                                 signal_order=test_signal))

        # not same order with same amount of word and brackets
        test_order = "signal the in order expected"
        test_signal = "expected order in the signal {{ variable }}"
        self.assertFalse(OrderAnalyser.is_ordered_strict_matching(user_order=test_order,
                                                                  signal_order=test_signal))

        # not same order with same amount of word and brackets with words after bracket
        test_order = "word expected order in the signal variable_to_catch with"
        test_signal = "expected order in the signal {{ variable }} with word"

        self.assertFalse(OrderAnalyser.is_ordered_strict_matching(user_order=test_order,
                                                                  signal_order=test_signal))

    def test_is_order_matching_signal(self):
        # Note: This is a private method, most of the test use cases are covered by `test_get_matching_synapse`

        # all lowercase
        test_order = "expected order in the signal"
        signal1 = Signal(name="order", parameters="expected order in the signal")
        test_signal = signal1
        self.assertTrue(OrderAnalyser.is_order_matching_signal(user_order=test_order,
                                                               signal=test_signal))

        # with uppercase
        test_order = "Expected Order In The Signal"
        test_signal = signal1
        self.assertTrue(OrderAnalyser.is_order_matching_signal(user_order=test_order,
                                                               signal=test_signal))

    def test_override_order_with_correction(self):
        # test with provided correction
        order = "this is my test"
        stt_correction = [{
            "input": "test",
            "output": "order"
        }]
        expected_output = "this is my order"
        new_order = OrderAnalyser.override_order_with_correction(order=order, stt_correction=stt_correction)
        self.assertEqual(new_order, expected_output)

        # missing key input
        order = "this is my test"
        stt_correction = [{
            "wrong_key": "test",
            "output": "order"
        }]
        expected_output = "this is my test"
        new_order = OrderAnalyser.override_order_with_correction(order=order, stt_correction=stt_correction)
        self.assertEqual(new_order, expected_output)

    def test_override_stt_correction_list(self):
        # test override
        list_stt_to_override = [
            {"input": "value1", "output": "value2"}
        ]

        overriding_list = [
            {"input": "value1", "output": "overridden"}
        ]

        expected_list = [
            {"input": "value1", "output": "overridden"}
        ]

        self.assertListEqual(expected_list, OrderAnalyser.override_stt_correction_list(list_stt_to_override,
                                                                                       overriding_list))

        # stt correction added from the overriding_list
        list_stt_to_override = [
            {"input": "value1", "output": "value2"}
        ]

        overriding_list = [
            {"input": "value2", "output": "overridden"}
        ]

        expected_list = [
            {"input": "value1", "output": "value2"},
            {"input": "value2", "output": "overridden"}
        ]

        self.assertListEqual(expected_list, OrderAnalyser.override_stt_correction_list(list_stt_to_override,
                                                                                       overriding_list))

    def test_order_correction(self):
        # test with only stt-correction
        testing_order = "thus is my test"

        signal_parameter = {
            "stt-correction": [
                {"input": "thus",
                 "output": "this"}
            ]
        }
        testing_signals = Signal(name="test",
                                 parameters=signal_parameter)

        expected_fixed_order = "this is my test"
        self.assertEqual(OrderAnalyser.order_correction(order=testing_order, signal=testing_signals),
                         expected_fixed_order)

        # test with both stt-correction and stt-correction-file
        testing_order = "thus is my test"

        signal_parameter = {
            "stt-correction": [
                {"input": "thus",
                 "output": "this"}
            ],
            "stt-correction-file": self.correction_file_to_test
        }
        testing_signals = Signal(name="test",
                                 parameters=signal_parameter)

        expected_fixed_order = "this is my order"
        self.assertEqual(OrderAnalyser.order_correction(order=testing_order, signal=testing_signals),
                         expected_fixed_order)

        # test with stt-correction that override stt-correction-file
        testing_order = "thus is my test"

        signal_parameter = {
            "stt-correction": [
                {"input": "test",
                 "output": "overridden"}
            ],
            "stt-correction-file": self.correction_file_to_test
        }
        testing_signals = Signal(name="test",
                                 parameters=signal_parameter)

        expected_fixed_order = "thus is my overridden"
        self.assertEqual(OrderAnalyser.order_correction(order=testing_order, signal=testing_signals),
                         expected_fixed_order)

        # test with stt-correction that override multiple words.
        testing_order = "thus is my test"

        signal_parameter = {
            "stt-correction": [
                {"input": "is my test",
                 "output": "is overridden"}
            ]
        }
        testing_signals = Signal(name="test",
                                 parameters=signal_parameter)

        expected_fixed_order = "thus is overridden"
        self.assertEqual(OrderAnalyser.order_correction(order=testing_order, signal=testing_signals),
                         expected_fixed_order)

        # test stt-correction that override one word with multiple words.
        testing_order = "thus is my test"

        signal_parameter = {
            "stt-correction": [
                {"input": "test",
                 "output": "is overridden"}
            ]
        }
        testing_signals = Signal(name="test",
                                 parameters=signal_parameter)

        expected_fixed_order = "thus is my is overridden"
        self.assertEqual(OrderAnalyser.order_correction(order=testing_order, signal=testing_signals),
                         expected_fixed_order)

        # test stt-correction that multiple words override files one word.
        testing_order = "thus is my test"

        signal_parameter = {
            "stt-correction": [
                {"input": "test",
                 "output": "the overridden"}
            ],
            "stt-correction-file": self.correction_file_to_test
        }
        testing_signals = Signal(name="test",
                                 parameters=signal_parameter)

        expected_fixed_order = "thus is my the overridden"
        self.assertEqual(OrderAnalyser.order_correction(order=testing_order, signal=testing_signals),
                         expected_fixed_order)

        # test stt-correction with multiple inputs words in file.
        testing_order = "hello the test"

        signal_parameter = {
            "stt-correction-file": self.correction_file_to_test
        }
        testing_signals = Signal(name="test",
                                 parameters=signal_parameter)

        expected_fixed_order = "i am order"
        self.assertEqual(OrderAnalyser.order_correction(order=testing_order, signal=testing_signals),
                         expected_fixed_order)

    def test_get_signal_order(self):
        signal1 = Signal(name="order", parameters="expected order in the signal")
        signal2 = Signal(name="order", parameters={"matching-type": "strict",
                                                   "text": "that is the sentence"})
        signal3 = Signal(name="order", parameters={"matching-type": "strict",
                                                   "fake-value": "that is the sentence"})

        # Signal order is a str
        expected_signal_order = "expected order in the signal"
        self.assertEqual(expected_signal_order, OrderAnalyser.get_signal_order(signal=signal1))

        # Signal order is a dict
        expected_signal_order = "that is the sentence"
        self.assertEqual(expected_signal_order, OrderAnalyser.get_signal_order(signal=signal2))

        # signal order is a dict and `text` element is missing
        expected_signal_order = ""  # not found !
        self.assertEqual(expected_signal_order, OrderAnalyser.get_signal_order(signal=signal3))

    def test_get_not_containing_words(self):
        signal1 = Signal(name="order", parameters={"matching-type": "not-contain", "excluded-words": ["that", "is"]})

        # get the correct list of excluded words
        expected_list_excluded_words = ["that", "is"]
        self.assertEqual(expected_list_excluded_words, OrderAnalyser.get_not_containing_words(signal=signal1))

        # assert it returns None when a str is provided and not a list
        signal1 = Signal(name="order", parameters={"matching-type": "not-contain", "excluded_words": "that"})
        self.assertIsNone(OrderAnalyser.get_not_containing_words(signal=signal1))

        # assert it returns None when `excluded_words` is not provided
        signal1 = Signal(name="order", parameters={"matching-type": "not-contain"})
        self.assertIsNone(OrderAnalyser.get_not_containing_words(signal=signal1))


if __name__ == '__main__':
    unittest.main()



import logging

import yaml
from ansible import context
from ansible.executor.task_queue_manager import TaskQueueManager
from ansible.inventory.manager import InventoryManager
from ansible.module_utils.common.collections import ImmutableDict
from ansible.parsing.dataloader import DataLoader
from ansible.playbook.play import Play
from ansible.vars.manager import VariableManager

from kalliope.core.NeuronModule import NeuronModule, MissingParameterException

logging.basicConfig()
logger = logging.getLogger("kalliope")


class Ansible_playbook(NeuronModule):
    def __init__(self, **kwargs):
        super(Ansible_playbook, self).__init__(**kwargs)

        self.task_file = kwargs.get('task_file', None)
        self.sudo = kwargs.get('sudo', False)
        self.sudo_user = kwargs.get('sudo_user', False)
        self.sudo_password = kwargs.get('sudo_password', False)

        # check if parameters have been provided
        if self._is_parameters_ok():
            # since the API is constructed for CLI it expects certain options to always be set in the context object
            context.CLIARGS = self._get_options()

            # initialize needed objects
            loader = DataLoader()

            passwords = {'vault_pass': self.sudo_password}

            inventory = InventoryManager(loader=loader, sources="localhost,")

            # variable manager takes care of merging all the different sources to give you a unified
            # view of variables available in each context
            variable_manager = VariableManager(loader=loader, inventory=inventory)
            variable_manager.set_inventory(inventory)

            playbooks = None
            with open(self.task_file, 'r') as stream:
                try:
                    playbooks = yaml.full_load(stream)
                except yaml.YAMLError as exc:
                    logger.debug("Ansibe playbook error: {}".format(exc))

            if playbooks is not None:
                # force usage of python 3 interpreter
                playbooks[0].setdefault("vars", {})
                playbooks[0]["vars"]["ansible_python_interpreter"] = "/usr/bin/python3"

                play = Play().load(playbooks[0], variable_manager=variable_manager, loader=loader)

                # Run it - instantiate task queue manager, which takes care of forking and setting up all objects
                # to iterate over host list and tasks
                tqm = None
                try:
                    tqm = TaskQueueManager(
                        inventory=inventory,
                        variable_manager=variable_manager,
                        loader=loader,
                        passwords=passwords,
                        stdout_callback='default',
                        # Use our custom callback instead of the ``default`` callback plugin, which prints to stdout
                    )
                    tqm.run(play)  # most interesting data for a play is actually sent to the callback's methods
                finally:
                    # we always need to cleanup child procs and the structres we use to communicate with them
                    if tqm is not None:
                        tqm.cleanup()

    def _is_parameters_ok(self):
        if self.task_file is None:
            raise MissingParameterException("task_file parameter required")

        # check if the user want to use sudo for root privileges
        if self.sudo:
            # the user must set a login and password
            if not self.sudo_user:
                raise MissingParameterException("sudo_user parameter required with sudo True")
            if not self.sudo_password:
                raise MissingParameterException("sudo_password parameter required with sudo True")

        return True

    def _get_options(self):
        """
        Return a valid dict of option usable by Ansible depending on the sudo value if set
        :return: ImmutableDict
        """

        if self.sudo:
            options = ImmutableDict(connection='local', forks=100, become=True, become_method="sudo",
                                    become_user=self.sudo_user, check=False, listhosts=False, listtasks=False, listtags=False,
                                    syntax=False, module_path="", diff=False)
        else:
            options = ImmutableDict(connection='local', forks=100, become=None, become_method=None, become_user=None,
                                    check=False, listhosts=False, listtasks=False, listtags=False, syntax=False,
                                    module_path="", diff=False)

        logger.debug("Ansible options: %s" % str(options))
        return options



import threading
from threading import Thread
import logging
from kalliope import Utils, SettingLoader
from kalliope.stt import SpeechRecognizer
import speech_recognition as sr

logging.basicConfig()
logger = logging.getLogger("kalliope")

class SpeechRecognition(Thread):

    def __init__(self, audio_file=None):
        """
        Thread used to process n audio file and pass it to a callback method
        """
        super(SpeechRecognition, self).__init__()
        self.callback = None
        self.audio_stream = None
        # get global configuration
        sl = SettingLoader()
        self.settings = sl.settings
        self.recognizer = SpeechRecognizer.ResponsiveRecognizer(multiplier=self.settings.options.recognizer_multiplier, 
                                                                energy_ratio=self.settings.options.recognizer_energy_ratio,
                                                                recording_timeout=self.settings.options.recognizer_recording_timeout,
                                                                recording_timeout_with_silence=self.settings.options.recognizer_recording_timeout_with_silence)

        if audio_file is None:
            # audio file not set, we need to capture a sample from the microphone
            self.microphone = SpeechRecognizer.MutableMicrophone()

        else:
            # audio file provided
            with sr.AudioFile(audio_file) as source:
                self.audio_stream = self.recognizer.record(source) # read the entire audio file

    def run(self):
        """
        Start the thread that listen the microphone and then give the audio to the callback method
        """
        if self.audio_stream is None:
            Utils.print_success("Say something!")
            with self.microphone as source:
                self.audio_stream = self.recognizer.listen(source)

        self.callback(self.recognizer, self.audio_stream)

    def start_processing(self):
        """
        A method to start the thread
        """
        self.start()

    def set_callback(self, callback):
        """
        set the callback method that will receive the audio stream caught by the microphone
        :param callback: callback method
        :return:
        """
        self.callback = callback
        



import re
from packaging import version
from kalliope._version import version_str
import requests

from kalliope.core import NeuronModule

KALLIOPE_PROJECT_MASTER_VERSION_URL = "https://raw.githubusercontent.com/kalliope-project/kalliope" \
                                      "/master/kalliope/_version.py"


class Kalliope_version(NeuronModule):

    def __init__(self, **kwargs):
        super(Kalliope_version, self).__init__(**kwargs)
        new_version_available = False
        last_master_version = None
        # get the last version online
        response = requests.get(KALLIOPE_PROJECT_MASTER_VERSION_URL)

        regex_version = r"(\d\.\d(\.\d)?(\.\d)?)"

        version_search = re.search(regex_version, response.text)
        if version_search:
            last_master_version = version_search.group(1)

        current_version = version_str

        if last_master_version:
            if version.parse(current_version) < version.parse(last_master_version):
                new_version_available = True

        message = {
            "current_version": current_version,
            "new_version_available": new_version_available,
            "last_master_version": last_master_version
        }

        self.say(message)



# Copyright 2017 Mycroft AI Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""
All credits for this code goes to the mycroft project.
Most of the code is untouched, some functions were removed because those are not necessary for our purpose. 
"""
import audioop
from time import sleep
import re
import collections
import json
import os
import pyaudio
import speech_recognition

from speech_recognition import (
    Microphone,
    AudioSource,
    AudioData
)


class MutableStream:
    def __init__(self, wrapped_stream, format):
        assert wrapped_stream is not None
        self.wrapped_stream = wrapped_stream

        self.SAMPLE_WIDTH = pyaudio.get_sample_size(format)
        self.muted_buffer = b''.join([b'\x00' * self.SAMPLE_WIDTH])


    def read(self, size, of_exc=False):
        """
            Read data from stream.

            Arguments:
                size (int): Number of bytes to read
                of_exc (bool): flag determining if the audio producer thread
                               should throw IOError at overflows.

            Returns:
                Data read from device
        """
        frames = collections.deque()
        remaining = size
        while remaining > 0:
            to_read = min(self.wrapped_stream.get_read_available(), remaining)
            if to_read == 0:
                sleep(.01)
                continue

            result = self.wrapped_stream.read(to_read,
                                              exception_on_overflow=of_exc)
            frames.append(result)
            remaining -= to_read

        input_latency = self.wrapped_stream.get_input_latency()
        audio = b"".join(list(frames))
        return audio

    def close(self):
        self.wrapped_stream.close()
        self.wrapped_stream = None

    def is_stopped(self):
        return self.wrapped_stream.is_stopped()

    def stop_stream(self):
        return self.wrapped_stream.stop_stream()


class MutableMicrophone(Microphone):
    def __init__(self, device_index=None, sample_rate=16000, 
                 chunk_size=1024):

        Microphone.__init__(self, device_index=device_index, 
                            sample_rate=sample_rate, chunk_size=chunk_size)
    def __enter__(self):    
        assert self.stream is None, \
            "This audio source is already inside a context manager"
        self.audio = pyaudio.PyAudio()
        self.stream = MutableStream(self.audio.open(
                                    input_device_index=self.device_index, 
                                    channels=1,
                                    format=self.format, 
                                    rate=self.SAMPLE_RATE,
                                    frames_per_buffer=self.CHUNK,
                                    input=True,  # stream is an input stream
                                    ), self.format)

        return self

    def __exit__(self, exc_type, exc_value, traceback):
        if not self.stream.is_stopped():
            self.stream.stop_stream()
        self.stream.close()
        self.stream = None
        self.audio.terminate()


def get_silence(num_bytes):
    return b'\0' * num_bytes

class NoiseTracker:
    """Noise tracker, used to deterimine if an audio utterance is complete.

    The current implementation expects a number of loud chunks (not necessary
    in one continous sequence) followed by a short period of continous quiet
    audio data to be considered complete.

    Arguments:
        minimum (int): lower noise level will be threshold for "quiet" level
        maximum (int): ceiling of noise level
        sec_per_buffer (float): the length of each buffer used when updating
                                the tracker
        loud_time_limit (float): time in seconds of low noise to be considered
                                 a complete sentence
        silence_time_limit (float): time limit for silence to abort sentence
        silence_after_loud (float): time of silence to finalize the sentence.
                                    default 0.25 seconds.
    """
    def __init__(self, minimum, maximum, sec_per_buffer, loud_time_limit,
                 silence_time_limit, silence_after_loud_time=0.25):
        self.min_level = minimum
        self.max_level = maximum
        self.sec_per_buffer = sec_per_buffer

        self.num_loud_chunks = 0
        self.level = 0

        # Smallest number of loud chunks required to return loud enough
        self.min_loud_chunks = int(loud_time_limit / sec_per_buffer)

        self.max_silence_duration = silence_time_limit
        self.silence_duration = 0

        # time of quite period after long enough loud data to consider the
        # sentence complete
        self.silence_after_loud = silence_after_loud_time

        # Constants
        self.increase_multiplier = 200
        self.decrease_multiplier = 100

    def _increase_noise(self):
        """Bumps the current level.

        Modifies the noise level with a factor depending in the buffer length.
        """
        if self.level < self.max_level:
            self.level += self.increase_multiplier * self.sec_per_buffer

    def _decrease_noise(self):
        """Decrease the current level.

        Modifies the noise level with a factor depending in the buffer length.
        """
        if self.level > self.min_level:
            self.level -= self.decrease_multiplier * self.sec_per_buffer

    def update(self, is_loud):
        """Update the tracking. with either a loud chunk or a quiet chunk.

        Arguments:
            is_loud: True if a loud chunk should be registered
                     False if a quiet chunk should be registered
        """
        if is_loud:
            self._increase_noise()
            self.num_loud_chunks += 1
        else:
            self._decrease_noise()
        # Update duration of energy under the threshold level
        if self._quiet_enough():
            self.silence_duration += self.sec_per_buffer
        else:  # Reset silence duration
            self.silence_duration = 0

    def _loud_enough(self):
        """Check if the noise loudness criteria is fulfilled.

        The noise is considered loud enough if it's been over the threshold
        for a certain number of chunks (accumulated, not in a row).
        """
        return self.num_loud_chunks > self.min_loud_chunks

    def _quiet_enough(self):
        """Check if the noise quietness criteria is fulfilled.

        The quiet level is instant and will return True if the level is lower
        or equal to the minimum noise level.
        """
        return self.level <= self.min_level

    def recording_complete(self):
        """Has the end creteria for the recording been met.

        If the noise level has decresed from a loud level to a low level
        the user has stopped speaking.

        Alternatively if a lot of silence was recorded without detecting
        a loud enough phrase.
        """
        too_much_silence = (self.silence_duration > self.max_silence_duration)
        return ((self._quiet_enough() and
                 self.silence_duration > self.silence_after_loud) and
                (self._loud_enough() or too_much_silence))

class ResponsiveRecognizer(speech_recognition.Recognizer):
    # The minimum seconds of noise before a
    # phrase can be considered complete
    MIN_LOUD_SEC_PER_PHRASE = 0.5

    # The minimum seconds of silence required at the end
    # before a phrase will be considered complete
    MIN_SILENCE_AT_END = 0.25

    # The maximum seconds a phrase can be recorded,
    # provided there is noise the entire time
    #RECORDING_TIMEOUT = 15.0

    # The maximum time it will continue to record silence
    # when not enough noise has been detected
    #RECORDING_TIMEOUT_WITH_SILENCE = 2.5 # default 3.0

    def __init__(self, 
                 multiplier, 
                 energy_ratio,
                 recording_timeout,
                 recording_timeout_with_silence):

        self.overflow_exc = False

        speech_recognition.Recognizer.__init__(self)
        self.audio = pyaudio.PyAudio()

        self.multiplier = multiplier
        self.energy_ratio = energy_ratio

        # The maximum seconds a phrase can be recorded,
        # provided there is noise the entire time
        self.recording_timeout = recording_timeout
        
        # The maximum time it will continue to record silence
        # when not enough noise has been detected
        self.recording_timeout_with_silence = recording_timeout_with_silence

    def record_sound_chunk(self, source):
        return source.stream.read(source.CHUNK, self.overflow_exc)

    @staticmethod
    def calc_energy(sound_chunk, sample_width):
        return audioop.rms(sound_chunk, sample_width)

    def _record_phrase(self, source, sec_per_buffer):
        """Record an entire spoken phrase.

        Essentially, this code waits for a period of silence and then returns
        the audio.  If silence isn't detected, it will terminate and return
        a buffer of self.recording_timeout duration.

        Args:
            source (AudioSource):  Source producing the audio chunks
            sec_per_buffer (float):  Fractional number of seconds in each chunk
            stream (AudioStreamHandler): Stream target that will receive chunks
                                         of the utterance audio while it is
                                         being recorded.
            ww_frames (deque):  Frames of audio data from the last part of wake
                                word detection.

        Returns:
            bytearray: complete audio buffer recorded, including any
                       silence at the end of the user's utterance
        """
        noise_tracker = NoiseTracker(0, 25, sec_per_buffer,
                                     self.MIN_LOUD_SEC_PER_PHRASE,
                                     self.recording_timeout_with_silence)

        # Maximum number of chunks to record before timing out
        max_chunks = int(self.recording_timeout / sec_per_buffer)
        num_chunks = 0

        # bytearray to store audio in, initialized with a single sample of
        # silence.
        byte_data = get_silence(source.SAMPLE_WIDTH)

        phrase_complete = False
        while num_chunks < max_chunks and not phrase_complete:
            chunk = self.record_sound_chunk(source)
            byte_data += chunk
            num_chunks += 1

            energy = self.calc_energy(chunk, source.SAMPLE_WIDTH)
            test_threshold = self.energy_threshold * self.multiplier
            is_loud = energy > test_threshold
            noise_tracker.update(is_loud)
            if not is_loud:
                self._adjust_threshold(energy, sec_per_buffer)

            # The phrase is complete if the noise_tracker end of sentence
            # criteria is met or if the  top-button is pressed
            phrase_complete = (noise_tracker.recording_complete())
            
        return byte_data
    
    @staticmethod
    def _create_audio_data(raw_data, source):
        """
        Constructs an AudioData instance with the same parameters
        as the source and the specified frame_data
        """
        return AudioData(raw_data, source.SAMPLE_RATE, source.SAMPLE_WIDTH)

    def listen(self, source):
        """Listens for chunks of audio that Mycroft should perform STT on.

        This will listen continuously for a wake-up-word, then return the
        audio chunk containing the spoken phrase that comes immediately
        afterwards.

        Args:
            source (AudioSource):  Source producing the audio chunks
            emitter (EventEmitter): Emitter for notifications of when recording
                                    begins and ends.

        Returns:
            AudioData: audio with the user's utterance, minus the wake-up-word
        """
        assert isinstance(source, AudioSource), "Source must be an AudioSource"

        #        bytes_per_sec = source.SAMPLE_RATE * source.SAMPLE_WIDTH
        sec_per_buffer = float(source.CHUNK) / source.SAMPLE_RATE

        # Every time a new 'listen()' request begins, reset the threshold
        # used for silence detection.  This is as good of a reset point as
        # any, as we expect the user and Mycroft to not be talking.
        # NOTE: adjust_for_ambient_noise() doc claims it will stop early if
        #       speech is detected, but there is no code to actually do that.
        
        self.adjust_for_ambient_noise(source, 0.1)
        frame_data = self._record_phrase(source, sec_per_buffer)
        audio_data = self._create_audio_data(frame_data, source)

        return audio_data

    def _adjust_threshold(self, energy, seconds_per_buffer):
        if self.dynamic_energy_threshold and energy > 0:
            # account for different chunk sizes and rates
            damping = (
                self.dynamic_energy_adjustment_damping ** seconds_per_buffer)
            target_energy = energy * self.energy_ratio
            self.energy_threshold = (
                self.energy_threshold * damping +
                target_energy * (1 - damping))



# coding: utf8
import collections
from collections import Counter
import six
import yaml
from jinja2 import Template

from kalliope.core.NeuronParameterLoader import NeuronParameterLoader

from kalliope.core.Models.MatchedSynapse import MatchedSynapse
from kalliope.core.Utils.Utils import Utils
from kalliope.core.ConfigurationManager import SettingLoader

import logging

logging.basicConfig()
logger = logging.getLogger("kalliope")


class OrderAnalyser:
    """
    This Class is used to get a list of synapses that match a given Spoken order
    """

    brain = None
    settings = None

    @classmethod
    def __init__(cls):
        cls.settings = SettingLoader().settings

    @classmethod
    def get_matching_synapse(cls, order, brain=None):
        """
        Return the list of matching synapses from the given order
        :param order: The user order
        :param brain: The loaded brain
        :return: The List of synapses matching the given order
        """
        cls.brain = brain
        logger.debug("[OrderAnalyser] Received order: %s" % order)
        if isinstance(order, six.binary_type):
            order = order.decode('utf-8')

        # We use a namedtuple to associate the synapse and the signal of the synapse
        synapse_order_tuple = collections.namedtuple('tuple_synapse_matchingOrder',
                                                     ['synapse', 'matched_order', 'user_order'])

        # if the received order is None we can stop the process immediately
        if order is None:
            return list()

        # test each synapse from the brain
        list_match_synapse = cls.get_list_match_synapse(order, synapse_order_tuple)

        # create a list of MatchedSynapse from the tuple list
        list_synapse_to_process = cls.get_list_synapses_to_process(list_match_synapse)

        return list_synapse_to_process

    @classmethod
    def get_list_synapses_to_process(cls, list_match_synapse):
        list_synapse_to_process = list()
        for tuple_el in list_match_synapse:
            new_matching_synapse = MatchedSynapse(matched_synapse=tuple_el.synapse,
                                                  matched_order=tuple_el.matched_order,
                                                  user_order=tuple_el.user_order)
            list_synapse_to_process.append(new_matching_synapse)
        return list_synapse_to_process

    @classmethod
    def get_list_match_synapse(cls, order, synapse_order_tuple):
        list_match_synapse = list()
        order = order.lower()  # Needed for STT Correction

        for synapse in cls.brain.synapses:
            if synapse.enabled:
                for signal in synapse.signals:
                    logger.debug("[OrderAnalyser] Testing Synapse name %s" % synapse.name)
                    # we are only concerned by synapse with a order type of signal
                    if signal.name == "order":
                        # Check if signal is matching the order and parameters.
                        if cls.is_order_matching_signal(user_order=order,
                                                        signal=signal):
                            order = cls.order_correction(order=order,
                                                         signal=signal)
                            logger.debug("Order found! Run synapse name: %s" % synapse.name)
                            Utils.print_success("Order matched in the brain. Running synapse \"%s\"" % synapse.name)
                            list_match_synapse.append(synapse_order_tuple(synapse=synapse,
                                                                          matched_order=cls.get_signal_order(signal),
                                                                          user_order=order))
                            # we matched this synapse with an order, don't need to check another
                            break
        return list_match_synapse

    @classmethod
    def order_correction(cls, order, signal):
        stt_correction_list = list()
        stt_correction_file_path = cls.get_stt_correction_file_path(signal)
        stt_correction = cls.get_stt_correction(signal)

        if stt_correction_file_path is not None:
            stt_correction_list = cls.load_stt_correction_file(stt_correction_file_path)
        if stt_correction is not None:
            stt_correction_list = cls.override_stt_correction_list(stt_correction_list, stt_correction)
        if stt_correction_list:
            order = cls.override_order_with_correction(order, stt_correction_list)
        return order

    @staticmethod
    def get_stt_correction(signal):
        stt_correction = None
        try:
            stt_correction = signal.parameters["stt-correction"]
            logger.debug("[OrderAnalyser] stt-correction provided by user")
        except KeyError:
            logger.debug("[OrderAnalyser] No stt-correction provided")
        finally:
            return stt_correction

    @staticmethod
    def get_not_containing_words(signal):
        not_containing_words = None
        try:
            not_containing_words = signal.parameters['excluded-words']
            if isinstance(not_containing_words, str):
                logger.debug("[OrderAnalyser] not contain words should be a list not a string.")
                not_containing_words = None
                raise KeyError
            logger.debug("[OrderAnalyser] not-contain provided by user : %s" % not_containing_words)
        except KeyError:
            logger.debug("[OrderAnalyser] No excluded-words provided, change expected_matching_type to normal")
        return not_containing_words

    @staticmethod
    def get_stt_correction_file_path(signal):
        stt_correction_file_path = None
        try:
            stt_correction_file_path = signal.parameters["stt-correction-file"]
            logger.debug("[OrderAnalyser] stt-correction-file provided by user")
        except KeyError:
            logger.debug("[OrderAnalyser] No stt-correction-file provided")
        finally:
            return stt_correction_file_path

    @staticmethod
    def get_matching_type(signal):
        expected_matching_type = "normal"
        try:
            expected_matching_type = signal.parameters["matching-type"]
        except KeyError:
            logger.debug("[OrderAnalyser] Warning, missing parameter 'matching-type' in order. "
                         "Fallback to 'normal'")
        finally:
            return expected_matching_type

    @staticmethod
    def _get_split_order_without_bracket(order):
        """
        Get an order with bracket inside like: "hello my name is {{ name }}.
        return a list of string without bracket like ["hello", "my", "name", "is"]
        :param order: sentence to split
        :return: list of string without bracket
        """

        matches = Utils.find_all_matching_brackets(order)
        for match in matches:
            order = order.replace(match, "")
        # then split
        split_order = order.split()
        return split_order

    @classmethod
    def is_not_contain_matching(cls, user_order, signal_order, not_containing_words):
        """
        True if :
            - all word in the user_order are present in the signal_order
            - if not containing words in the order
        :param user_order: order from the user
        :param signal_order: order in the signal
        :param not_containing_words: list of words which are not present in the order
        :return: Boolean
        """
        logger.debug(
            "[OrderAnalyser] is_not_contain_matching called with user_order: %s, signal_order: %s and should not contains %s"
            % (user_order, signal_order, not_containing_words))

        # Check that normal matching is valid otherwise returns False
        if not cls.is_normal_matching(user_order, signal_order):
            return False
        for m in not_containing_words:
            if m in user_order.split():
                return False
        return True

    @classmethod
    def is_normal_matching(cls, user_order, signal_order):
        """
        True if :
            - all word in the user_order are present in the signal_order
        :param user_order: order from the user
        :param signal_order: order in the signal
        :return: Boolean
        """
        logger.debug("[OrderAnalyser] is_normal_matching called with user_order: %s, signal_order: %s" % (user_order,
                                                                                                          signal_order))
        if not signal_order: return False

        split_user_order = user_order.split()
        split_signal_order_without_brackets = cls._get_split_order_without_bracket(signal_order)

        c1, c2 = Counter(split_signal_order_without_brackets), Counter(split_user_order)
        for k, n in c1.items():
            if n > c2[k]:
                return False
        return True

    @classmethod
    def is_strict_matching(cls, user_order, signal_order):
        """
        True if :
            - all word in the user_order are present in the signal_order
            - no additional word
        :param user_order: order from the user
        :param signal_order: order in the signal
        :return: Boolean
        """
        logger.debug("[OrderAnalyser] is_strict_matching called with user_order: %s, signal_order: %s" % (user_order,
                                                                                                          signal_order))
        if cls.is_normal_matching(user_order=user_order, signal_order=signal_order):

            # if the signal order contains bracket, we need to instantiate it with loaded parameters from the user order
            if Utils.is_containing_bracket(signal_order):
                signal_order = cls._get_instantiated_order_signal_from_user_order(signal_order, user_order)

            split_user_order = user_order.split()
            split_instantiated_signal = signal_order.split()

            if len(split_user_order) == len(split_instantiated_signal):
                return True

        return False

    @classmethod
    def is_ordered_strict_matching(cls, user_order, signal_order):
        """
       True if :
            - all word in the user_order are present in the signal_order
            - no additional word
            - same order as word present in signal_order
        :param user_order: order from the user
        :param signal_order: order in the signal
        :return: Boolean
        """
        logger.debug(
            "[OrderAnalyser] ordered_strict_matching called with user_order: %s, signal_order: %s" % (user_order,
                                                                                                      signal_order))
        if cls.is_normal_matching(user_order=user_order, signal_order=signal_order) and \
                cls.is_strict_matching(user_order=user_order, signal_order=signal_order):
            # if the signal order contains bracket, we need to instantiate it with loaded parameters from the user order
            if Utils.is_containing_bracket(signal_order):
                signal_order = cls._get_instantiated_order_signal_from_user_order(signal_order, user_order)

            split_user_order = user_order.split()
            split_signal_order = signal_order.split()
            return split_user_order == split_signal_order

        return False

    @classmethod
    def is_order_matching_signal(cls, user_order, signal):
        """
        return True if the user_order matches the provided signal. False otherwise
        Note: it applies `stt-correction` and handles `not-contain` matching type.
        :param user_order: order from the user
        :param signal: The signal to compare with the user_order
        :return: True if the order match
        """
        matching_type_function = {
            "normal": cls.is_normal_matching,
            "strict": cls.is_strict_matching,
            "ordered-strict": cls.is_ordered_strict_matching,
            "not-contain": cls.is_not_contain_matching
        }

        expected_order_type = cls.get_matching_type(signal)

        signal_order = cls.get_signal_order(signal)
        user_order = cls.order_correction(user_order, signal)

        # Lowercase all incoming
        user_order = user_order.lower()
        signal_order = signal_order.lower()

        if expected_order_type in matching_type_function:
            if expected_order_type == "not-contain":
                not_containing_words = cls.get_not_containing_words(signal)
                return cls.is_not_contain_matching(user_order, signal_order, not_containing_words)
            else:
                return matching_type_function[expected_order_type](user_order, signal_order)
        else:
            return cls.is_normal_matching(user_order, signal_order)

    @classmethod
    def get_signal_order(cls, signal):
        signal_order = ""
        if isinstance(signal.parameters, str) or isinstance(signal.parameters, six.text_type):
            signal_order = signal.parameters
        if isinstance(signal.parameters, dict):
            try:
                signal_order = signal.parameters["text"]
            except KeyError:
                logger.debug("[OrderAnalyser] Warning, missing parameter 'text' in order. "
                             "Signal Order will be skipped")
        return signal_order

    @classmethod
    def _get_instantiated_order_signal_from_user_order(cls, signal_order, user_order):
        """
        return instantiated signal order with parameters loaded from the user order
        E.g:
        signal_order = "this is an {{ variable }}
        user_order = "this is an order"

        returned value is: "this is an order"

        :param user_order: the order from the user
        :param signal_order: the order with brackets from the synapse
        :return: jinja instantiated order from the signal
        """
        # get parameters
        parameters_from_user_order = NeuronParameterLoader.get_parameters(synapse_order=signal_order,
                                                                          user_order=user_order)
        # we load variables into the expected order from the signal
        t = Template(signal_order)
        signal_order = t.render(**parameters_from_user_order)

        return signal_order

    @classmethod
    def override_order_with_correction(cls, order, stt_correction):
        logger.debug("[OrderAnalyser] override_order_this_correction: stt_correction list: %s" % stt_correction)
        logger.debug("[OrderAnalyser] override_order_this_correction: order before correction: %s" % order)
        for correction in stt_correction:
            try:
                input_val = str(correction["input"]).lower()
                output_val = str(correction["output"]).lower()
            except KeyError as e:
                logger.debug("[OrderAnalyser] override_order_this_correction: "
                             "Missing key %s. Order will not be modified" % e)
                return order

            if str(input_val) in order:  # remove split for the case a correction contains more than one word
                logger.debug("[OrderAnalyser] STT override '%s' by '%s'" % (input_val, output_val))
                order = order.replace(input_val, output_val)

        return order

    @classmethod
    def load_stt_correction_file(cls, stt_correction_file):
        stt_correction_file_path = Utils.get_real_file_path(stt_correction_file)
        stt_correction_file = open(stt_correction_file_path, "r")
        stt_correction = yaml.full_load(stt_correction_file)

        return stt_correction

    @classmethod
    def override_stt_correction_list(cls, stt_correction_list_to_override, correction_list):
        # add non existing dict
        for correction_to_check in correction_list:
            if correction_to_check["input"] not in (x["input"] for x in stt_correction_list_to_override):
                stt_correction_list_to_override.append(correction_to_check)

        # override dict with same input
        for correction_to_override in stt_correction_list_to_override:
            for correction_to_check in correction_list:
                if correction_to_check["input"] == correction_to_override["input"]:
                    correction_to_override["output"] = correction_to_check["output"]
                    break

        return stt_correction_list_to_override



import logging

from kalliope.core import NeuronModule
import alsaaudio

from kalliope.core.NeuronModule import InvalidParameterException

logging.basicConfig()
logger = logging.getLogger("kalliope")


class SoundManager(object):

    try:
        m = alsaaudio.Mixer()
    except alsaaudio.ALSAAudioError:
        # no master, we are on a Rpi
        try:
            m = alsaaudio.Mixer("PCM")
        except alsaaudio.ALSAAudioError:
            # no audio config at all
            m = None

    @classmethod
    def set_volume(cls, volume_level):
        if cls.m is not None:
            cls.m.setvolume(int(volume_level))

    @classmethod
    def get_volume(cls):
        if cls.m is not None:
            vol = cls.m.getvolume()
            return int(vol[0])
        return None


class Volume(NeuronModule):

    def __init__(self, **kwargs):
        super(Volume, self).__init__(**kwargs)

        self.level = kwargs.get('level', None)
        self.action = kwargs.get('action', "set")  # can be set, raise or lower

        # check parameters
        if self._is_parameters_ok():
            if self.action == "set":
                logger.debug("[Volume] set volume to: {}".format(self.level))
                SoundManager.set_volume(self.level)
            if self.action == "raise":
                current_level = SoundManager.get_volume()
                level_to_set = self.level + current_level
                if level_to_set > 100:
                    level_to_set = 100
                logger.debug("[Volume] set volume to: {}".format(level_to_set))
                SoundManager.set_volume(level_to_set)
            if self.action == "lower":
                current_level = SoundManager.get_volume()
                level_to_set = current_level - self.level
                if level_to_set < 0:
                    level_to_set = 0
                logger.debug("[Volume] set volume to: {}".format(level_to_set))
                SoundManager.set_volume(level_to_set)

            message = {
                "asked_level": self.level,
                "asked_action": self.action,
                "current_level": SoundManager.get_volume()
            }
            self.say(message)

    def _is_parameters_ok(self):
        if self.level is None:
            raise InvalidParameterException("[Volume] level need to be set")
        try:
            self.level = int(self.level)
        except ValueError:
            raise InvalidParameterException("[Volume] level '{}' is not a valid integer".format(self.level))
        if self.level < 0 or self.level > 100:
            raise InvalidParameterException("[Volume] level need to be placed between 0 and 100")
        if self.action not in ["set", "raise", "lower"]:
            raise InvalidParameterException("[Volume] action can be 'set', 'raise' or 'lower'")
        return True



import logging
import threading

from flask import jsonify
from flask import request
from flask_cors import CORS

from kalliope._version import version_str
from kalliope.core.ConfigurationManager import SettingLoader
from kalliope.core.RestAPI.utils import requires_auth
from kalliope.core.RestAPI.views.neurons_view import NeuronsView
from kalliope.core.RestAPI.views.settings_views import SettingsView
from kalliope.core.RestAPI.views.synapses_views import SynapsesView
from kalliope.core.Utils.FileManager import FileManager
from gevent.pywsgi import WSGIServer

logging.basicConfig()
logger = logging.getLogger("kalliope")

UPLOAD_FOLDER = '/tmp/kalliope/tmp_uploaded_audio'
ALLOWED_EXTENSIONS = {'wav'}


class FlaskAPI(threading.Thread):
    def __init__(self, app, port=5000, brain=None, allowed_cors_origin=False):
        """

        :param app: Flask API
        :param port: Port to listen
        :param brain: Brain object
        :type brain: Brain
        """
        super(FlaskAPI, self).__init__()
        self.app = app
        self.port = port
        self.brain = brain
        self.allowed_cors_origin = allowed_cors_origin

        # get current settings
        sl = SettingLoader()
        self.settings = sl.settings

        # configure the upload folder
        app.config['UPLOAD_FOLDER'] = UPLOAD_FOLDER
        # create the temp folder
        FileManager.create_directory(UPLOAD_FOLDER)

        # Flask configuration remove default Flask behaviour to encode to ASCII
        self.app.url_map.strict_slashes = False
        self.app.config['JSON_AS_ASCII'] = False

        if self.allowed_cors_origin is not False:
            CORS(app, resources={r"/*": {"origins": allowed_cors_origin}}, supports_credentials=True)

        # Add routing rules
        self.app.add_url_rule('/', view_func=self.get_main_page, methods=['GET'])
        self.app.add_url_rule('/shutdown/', view_func=self.shutdown_server, methods=['POST'])

        # Register blue prints
        self.synapses_blueprint = SynapsesView('synapses',
                                               __name__,
                                               app=self.app,
                                               brain=self.brain,
                                               settings=self.settings)
        self.app.register_blueprint(self.synapses_blueprint)
        self.settings_blueprint = SettingsView('settings',
                                               __name__,
                                               app=self.app,
                                               brain=self.brain,
                                               settings=self.settings)
        self.app.register_blueprint(self.settings_blueprint)
        self.neurons_blueprint = NeuronsView('neurons',
                                             __name__,
                                             app=self.app,
                                             brain=self.brain,
                                             settings=self.settings)
        self.app.register_blueprint(self.neurons_blueprint)

    def run(self):
        http_server = WSGIServer(('', 5000), self.app)
        http_server.serve_forever()

    @requires_auth
    def get_main_page(self):
        logger.debug("[FlaskAPI] get_main_page")
        data = {
            "Kalliope version": "%s" % version_str
        }
        return jsonify(data), 200

    @requires_auth
    def shutdown_server(self):
        func = request.environ.get('werkzeug.server.shutdown')
        if func is None:
            raise RuntimeError('Not running with the Werkzeug Server')
        func()
        return "Shutting down..."



import os
import unittest

import mock

from kalliope.core.NeuronModule import MissingParameterException
from kalliope.neurons.say.say import Say


class TestSay(unittest.TestCase):

    def setUp(self):
        self.message = "message"
        self.random = "random"
        self.pico_say_method_path = "kalliope.tts.pico2wave.pico2wave.Pico2wave.say"

    def testParameters(self):
        def run_test(parameters_to_test):
            with self.assertRaises(MissingParameterException):
                Say(**parameters_to_test)

        # empty
        parameters = dict()
        run_test(parameters)

        # missing message
        parameters = {
            "random": self.random
        }
        run_test(parameters)

    def test_say(self):
        # single message
        parameters = {
            "message": "test message"
        }
        with mock.patch(self.pico_say_method_path) as mock_tts:
            Say(**parameters)
            mock_tts.assert_called_once_with("test message")

        # template
        template_path = "test_say_neuron_template.j2"
        current_path = os.getcwd()
        if "tests" not in current_path:
            template_path = "kalliope/neurons/say/tests/test_say_neuron_template.j2"

        parameters = {
            "file_template": template_path
        }
        with mock.patch(self.pico_say_method_path) as mock_tts:
            Say(**parameters)
            mock_tts.assert_called_once_with("hello sir")

    # TODO fix this one. Working fine when running alone. Not working when running full test
    # def test_synapse_with_say(self):
    #     neuron1 = Neuron(name='say', parameters={'message': 'I say hello to {{ variable }}'})
    #     signal1 = Signal(name="order", parameters="hello {{ variable }}")
    #     synapse1 = Synapse(name="Synapse1", neurons=[neuron1], signals=[signal1])
    #
    #     all_synapse_list = [synapse1]
    #     brain_test = Brain(synapses=all_synapse_list)
    #
    #     with mock.patch(self.pico_say_method_path) as mock_tts:
    #         SynapseLauncher.run_matching_synapse_from_order(order_to_process="hello world",
    #                                                         brain=brain_test,
    #                                                         settings=None)
    #         mock_tts.assert_called_once_with("I say hello to world")
    #
    # def test_synapse_with_say_and_template(self):
    #     template_path = "test_say_neuron_template_with_variable.j2"
    #     current_path = os.getcwd()
    #     if "tests" not in current_path:
    #         template_path = "kalliope/neurons/say/tests/test_say_neuron_template_with_variable.j2"
    #
    #     neuron1 = Neuron(name='say', parameters={'file_template': template_path,
    #                                              'parameters': {"variable": "{{ variable }}"}})
    #     signal1 = Signal(name="order", parameters="hello {{ variable }}")
    #     synapse1 = Synapse(name="Synapse1", neurons=[neuron1], signals=[signal1])
    #     all_synapse_list = [synapse1]
    #     brain_test = Brain(synapses=all_synapse_list)
    #     with mock.patch(self.pico_say_method_path) as mock_tts:
    #         SynapseLauncher.run_matching_synapse_from_order(order_to_process="hello world",
    #                                                         brain=brain_test,
    #                                                         settings=None)
    #         mock_tts.assert_called_once_with("I say hello to world")



import logging
from threading import Thread

import requests

GA_TRACKING_ID = "UA-124800612-1"

logging.basicConfig()
logger = logging.getLogger("kalliope")
logger.setLevel(logging.DEBUG)


class GoogleTracking(Thread):
    """
    send hit to Google Analytics
    allow to anonymously evaluate the global usage of Kalliope app by users
    """

    def __init__(self, **kwargs):
        super(GoogleTracking, self).__init__()
        self.category = kwargs.get("category")
        self.action = kwargs.get("action")
        self.label = kwargs.get("label", None)
        self.value = kwargs.get("value", 0)
        self.kalliope_version = kwargs.get("kalliope_version", 0)

    def run(self):
        self.track_event(self.kalliope_version, self.category, self.action, self.label, self.value)

    @staticmethod
    def track_event(cid, kalliope_version, category, action, label=None, value=0):
        # allowed parameters: https://developers.google.com/analytics/devguides/collection/protocol/v1/parameters
        data = {
            'v': '1',  # API Version.
            'tid': GA_TRACKING_ID,  # Tracking ID / Property ID.
            'cid': cid,  # unique user id
            'an': "kalliope",
            'av': kalliope_version,
            'ds': 'api',
            't': 'event',  # Event hit type.
            'ec': category,  # Event category.
            'ea': action,  # Event action.
            'el': label,  # Event label.
            'ev': value,  # Event value, must be an integer
        }
        try:
            response = requests.post(
                'http://www.google-analytics.com/collect', data=data)

        # If the request fails, this will raise a RequestException.
            response.raise_for_status()

            logger.debug("[GoogleTracking] hit sent: %s" % response.status_code)
        except Exception as e:
            logger.debug("[GoogleTracking] fail to send data: %s" % e)




import inspect
import os
import shutil
import unittest

import mock

from kalliope import SettingLoader
from kalliope.core.ConfigurationManager import SettingEditor
from kalliope.core.Models.settings.Player import Player
from kalliope.core.Models.settings.Stt import Stt
from kalliope.core.Models.settings.Trigger import Trigger
from kalliope.core.Models.settings.Tts import Tts


class TestSettingEditor(unittest.TestCase):
    """
    Test class for the ~SettingEditor class and methods
    """

    def setUp(self):
        # get current script directory path. We are in /an/unknown/path/kalliope/core/tests
        cur_script_directory = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
        # get parent dir. Now we are in /an/unknown/path/kalliope
        root_dir = os.path.normpath(cur_script_directory + os.sep + os.pardir)

        self.settings_file_to_test = root_dir + os.sep + "Tests/settings/settings_test.yml"

        # Init the folders, otherwise it raises an exceptions
        os.makedirs("/tmp/kalliope/tests/kalliope_resources_dir/neurons")
        os.makedirs("/tmp/kalliope/tests/kalliope_resources_dir/stt")
        os.makedirs("/tmp/kalliope/tests/kalliope_resources_dir/tts")
        os.makedirs("/tmp/kalliope/tests/kalliope_resources_dir/trigger")

        self.sl = SettingLoader(file_path=self.settings_file_to_test)

    def tearDown(self):
        # Cleanup
        shutil.rmtree('/tmp/kalliope/tests/kalliope_resources_dir')

    def test_set_mute_status(self):
        with mock.patch("kalliope.core.ConfigurationManager.SettingLoader") as mock_setting_loader:
            mock_setting_loader.return_value(self.sl)
            SettingEditor.set_mute_status(mute=True)
            self.assertTrue(self.sl.settings.options.mute)

    def test_set_deaf_status(self):
        with mock.patch("kalliope.core.ConfigurationManager.SettingLoader") as mock_setting_loader:
            mock_setting_loader.return_value(self.sl)
            SettingEditor.set_deaf_status(mock.Mock(), deaf=False)
            self.assertFalse(self.sl.settings.options.deaf)

    def test_set_recognizer_multiplier(self):
        with mock.patch("kalliope.core.ConfigurationManager.SettingLoader") as mock_setting_loader:
            mock_setting_loader.return_value(self.sl)
            SettingEditor.set_recognizer_multiplier(600.0)
            self.assertEqual(600.0, self.sl.settings.options.recognizer_multiplier)

    def test_set_recognizer_energy_ratio(self):
        with mock.patch("kalliope.core.ConfigurationManager.SettingLoader") as mock_setting_loader:
            mock_setting_loader.return_value(self.sl)
            SettingEditor.set_recognizer_energy_ratio(600.0)
            self.assertEqual(600.0, self.sl.settings.options.recognizer_energy_ratio)
    
    def test_set_recognizer_recording_timeout(self):
        with mock.patch("kalliope.core.ConfigurationManager.SettingLoader") as mock_setting_loader:
            mock_setting_loader.return_value(self.sl)
            SettingEditor.set_recognizer_recording_timeout(600.0)
            self.assertEqual(600.0, self.sl.settings.options.recognizer_recording_timeout)

    def test_set_recognizer_recording_timeout_with_silence(self):
        with mock.patch("kalliope.core.ConfigurationManager.SettingLoader") as mock_setting_loader:
            mock_setting_loader.return_value(self.sl)
            SettingEditor.set_recognizer_recording_timeout_with_silence(600.0)
            self.assertEqual(600.0, self.sl.settings.options.recognizer_recording_timeout_with_silence)

    def test_set_default_player(self):
        default_name = "NamePlayer"
        with mock.patch("kalliope.core.ConfigurationManager.SettingLoader") as mock_setting_loader:
            mock_setting_loader.return_value(self.sl)
            SettingEditor.set_default_player(default_name)
            self.assertEqual("mplayer", self.sl.settings.default_player_name)  # not existing in the list, not updated !
            default_name = "pyalsaaudio"
            SettingEditor.set_default_player(default_name)
            self.assertEqual(default_name, self.sl.settings.default_player_name)  # Updated

    def test_set_players(self):
        new_player = Player(name="totoplayer", parameters={})
        with mock.patch("kalliope.core.ConfigurationManager.SettingLoader") as mock_setting_loader:
            mock_setting_loader.return_value(self.sl)
            SettingEditor.set_players(new_player)
            self.assertIn(new_player, self.sl.settings.players)

    def test_set_default_trigger(self):
        default_name = "NameTrigger"
        with mock.patch("kalliope.core.ConfigurationManager.SettingLoader") as mock_setting_loader:
            mock_setting_loader.return_value(self.sl)
            SettingEditor.set_default_trigger(default_name)
            self.assertEqual("snowboy",
                             self.sl.settings.default_trigger_name)  # not existing in the list, not updated !
            default_name = "snowboy"
            SettingEditor.set_default_trigger(default_name)
            self.assertEqual(default_name, self.sl.settings.default_trigger_name)

    def test_set_triggers(self):
        new_trigger = Trigger(name="tototrigger", parameters={})
        with mock.patch("kalliope.core.ConfigurationManager.SettingLoader") as mock_setting_loader:
            mock_setting_loader.return_value(self.sl)
            SettingEditor.set_trigger(new_trigger)
            self.assertIn(new_trigger, self.sl.settings.triggers)

    def test_set_default_stt(self):
        default_name = "NameStt"
        with mock.patch("kalliope.core.ConfigurationManager.SettingLoader") as mock_setting_loader:
            mock_setting_loader.return_value(self.sl)
            SettingEditor.set_default_stt(default_name)
            self.assertEqual("google", self.sl.settings.default_stt_name) # not updated because not in the list
            default_name = "google"
            SettingEditor.set_default_stt(default_name)
            self.assertEqual(default_name, self.sl.settings.default_stt_name) # updated

    def test_set_stts(self):
        new_stt = Stt(name="totoStt", parameters={})
        with mock.patch("kalliope.core.ConfigurationManager.SettingLoader") as mock_setting_loader:
            mock_setting_loader.return_value(self.sl)
            SettingEditor.set_stts(new_stt)
            self.assertIn(new_stt, self.sl.settings.stts)

    def test_set_default_tts(self):
        default_name = "NameTts"
        with mock.patch("kalliope.core.ConfigurationManager.SettingLoader") as mock_setting_loader:
            mock_setting_loader.return_value(self.sl)
            SettingEditor.set_default_tts(default_name)
            self.assertEqual("pico2wave", self.sl.settings.default_tts_name)
            default_name = "googletts"
            SettingEditor.set_default_tts(default_name)
            self.assertEqual(default_name, self.sl.settings.default_tts_name)

    def test_set_ttss(self):
        new_tts = Tts(name="totoTss", parameters={})
        with mock.patch("kalliope.core.ConfigurationManager.SettingLoader") as mock_setting_loader:
            mock_setting_loader.return_value(self.sl)
            SettingEditor.set_ttss(new_tts)
            self.assertIn(new_tts, self.sl.settings.ttss)

    def test_set_hooks(self):
        default_hooks = {"on_deaf": "randomSynapse"}
        with mock.patch("kalliope.core.ConfigurationManager.SettingLoader") as mock_setting_loader:
            mock_setting_loader.return_value(self.sl)
            SettingEditor.set_hooks(default_hooks)
            self.assertEqual("randomSynapse", self.sl.settings.hooks["on_deaf"])
            # self.assertTrue(set(default_hooks.items()).issubset(set(self.sl.settings.hooks))) # Not working for non hashable values

    def test_set_variabless(self):
        default_variables = {"coucou": "hello"}
        with mock.patch("kalliope.core.ConfigurationManager.SettingLoader") as mock_setting_loader:
            mock_setting_loader.return_value(self.sl)
            SettingEditor.set_variables(default_variables)
            self.assertEqual("hello", self.sl.settings.variables["coucou"])



import json
import unittest

from Tests.test_api.base import RestAPITestBase
from kalliope._version import version_str


class TestMainView(RestAPITestBase):

    def test_server_is_up_and_running(self):
        # response = urllib2.urlopen(self.get_server_url())
        response = self.client.get(self.get_server_url())
        self.assertEqual(response.status_code, 200)

    def test_get_main_page(self):
        url = self.get_server_url() + "/"
        response = self.client.get(url)
        expected_content = {
            "Kalliope version": "%s" % version_str
        }
        self.assertEqual(json.dumps(expected_content, sort_keys=True),
                         json.dumps(json.loads(response.get_data().decode('utf-8')), sort_keys=True))


if __name__ == '__main__':
    unittest.main()



import logging
from threading import Thread
from time import sleep

from kalliope.core.SignalModule import SignalModule
from kalliope.core.Cortex import Cortex
from kalliope.core.SynapseLauncher import SynapseLauncher

from kalliope.core.OrderListener import OrderListener

from kalliope import Utils, BrainLoader

from kalliope.core.TriggerLauncher import TriggerLauncher
from transitions import Machine

from kalliope.core.PlayerLauncher import PlayerLauncher

from kalliope.core.ConfigurationManager import SettingLoader

from kalliope.core.HookManager import HookManager

logging.basicConfig()
logger = logging.getLogger("kalliope")


class Order(SignalModule, Thread):

    states = ['init',
              'starting_trigger',
              'unpausing_trigger',
              'waiting_for_trigger_callback',
              'pausing_trigger',
              'start_order_listener',
              'waiting_for_order_listener_callback',
              'analysing_order']

    def __init__(self):
        super(SignalModule, self).__init__()
        Thread.__init__(self, name=Order)
        Utils.print_info('Starting order signal')
        # load settings and brain from singleton
        sl = SettingLoader()
        self.settings = sl.settings
        self.brain = BrainLoader().brain

        # keep in memory the order to process
        self.order_to_process = None

        # get the player instance
        self.player_instance = PlayerLauncher.get_player(settings=self.settings)

        # save an instance of the trigger
        self.trigger_instance = None
        self.trigger_callback_called = False

        # variable from notifications
        self.skip_trigger = False       # keep the status of the trigger, if true we can skip it in the statue machine
        self.counter_max_retry = 0      # 0 means disabled

        # save the current order listener
        self.order_listener = None
        self.order_listener_callback_called = False

        # Initialize the state machine
        self.machine = Machine(model=self, states=Order.states, initial='init', queued=True)

        # define transitions
        self.machine.add_transition('start_trigger', 'init', 'starting_trigger')
        self.machine.add_transition('unpause_trigger',  'analysing_order', 'unpausing_trigger')
        self.machine.add_transition('wait_trigger_callback', 'unpausing_trigger', 'waiting_for_trigger_callback')
        self.machine.add_transition('pause_trigger', 'waiting_for_trigger_callback', 'pausing_trigger')
        self.machine.add_transition('wait_for_order', 'pausing_trigger', 'waiting_for_order_listener_callback')
        self.machine.add_transition('analyse_order', 'waiting_for_order_listener_callback', 'analysing_order')
        self.machine.add_transition('start_order_listener', 'analysing_order', 'start_order_listener')

        self.machine.add_ordered_transitions()

        # add method which are called when changing state
        self.machine.on_enter_starting_trigger('start_trigger_process')
        self.machine.on_enter_unpausing_trigger('unpausing_trigger_process')
        self.machine.on_enter_waiting_for_trigger_callback('waiting_for_trigger_callback_thread')
        self.machine.on_enter_pausing_trigger('pause_trigger_process')
        self.machine.on_enter_start_order_listener('start_order_listener_thread')
        self.machine.on_enter_waiting_for_order_listener_callback('waiting_for_order_listener_callback_thread')
        self.machine.on_enter_analysing_order('analysing_order_thread')
        
    def run(self):
        # run hook on_start
        self.start_trigger()

    def start_trigger_process(self):
        """
        This function will start the trigger thread that listen for the hotword
        """
        logger.debug("[Order] Entering state: %s" % self.state)
        self.trigger_instance = TriggerLauncher.get_trigger(settings=self.settings, callback=self.trigger_callback)
        self.trigger_callback_called = False
        self.trigger_instance.daemon = True
        # Wait that the kalliope trigger is pronounced by the user
        self.trigger_instance.start()
        HookManager.on_start()
        self.next_state()

    def unpausing_trigger_process(self):    
        """ 
        If the trigger was in pause, this method will unpause it to listen again for the hotword    
        """ 
        logger.debug("Entering state: %s" % self.state) 
        HookManager.on_waiting_for_trigger()
        self.trigger_instance.unpause()  
        self.trigger_callback_called = False    
        self.next_state()

    def waiting_for_trigger_callback_thread(self):
        """
        Method to print in debug that the main process is waiting for a trigger detection
        """
        logger.debug("[Order] Entering state: %s" % self.state)
        if self.settings.options.deaf:  # the user asked to deaf inside the deaf neuron
            Utils.print_info("Kalliope is deaf")
            self.trigger_instance.pause()
        else:
            Utils.print_info("Waiting for trigger detection")
        # this loop is used to keep the main thread alive
        while not self.trigger_callback_called:
            sleep(0.1)
        self.next_state()

    def waiting_for_order_listener_callback_thread(self):
        """
        Method to print in debug that the main process is waiting for an order to analyse
        """
        logger.debug("[Order] Entering state: %s" % self.state)
        # this loop is used to keep the main thread alive
        while not self.order_listener_callback_called:
            sleep(0.1)
        # TODO on end listening here
        self.next_state()

    def trigger_callback(self):
        """
        we have detected the hotword, we can now pause the Trigger for a while
        The user can speak out loud his order during this time.
        """
        logger.debug("[Order] Trigger callback called, switching to the next state")
        self.trigger_callback_called = True

    def pause_trigger_process(self):
        """
        The trigger has been awaken, we pause it
        :return:
        """
        logger.debug("[Order] Entering state: %s" % self.state)
        self.trigger_instance.pause()
        # if here, then the trigger has been called and paused
        HookManager.on_triggered()
        self.next_state()

    def start_order_listener_thread(self):
        """
        Start the STT engine thread
        """
        logger.debug("[Order] Entering state: %s" % self.state)

        # start listening for an order
        HookManager.on_start_listening()
        self.order_listener_callback_called = False
        self.order_listener = OrderListener(callback=self.order_listener_callback)
        self.order_listener.daemon = True
        self.order_listener.start()
        self.next_state()

    def order_listener_callback(self, order):
        """
        Receive an order, try to retrieve it in the brain.yml to launch to attached plugins
        :param order: the sentence received
        :type order: str
        """
        logger.debug("[Order] Order listener callback called. Order to process: %s" % order)
        HookManager.on_stop_listening()
        self.order_to_process = order
        self.order_listener_callback_called = True
        # save in kalliope memory the last order
        Cortex.save('kalliope_last_order', order)

    def analysing_order_thread(self):
        """
        Start the order analyser with the caught order to process
        """
        if self.order_to_process is None or self.order_to_process == "":
            logger.debug("[Order] No audio caught from analysing_order_thread")
            HookManager.on_stt_error()
        else:
            logger.debug("[Order] order in analysing_order_thread %s" % self.order_to_process)
            SynapseLauncher.run_matching_synapse_from_order(self.order_to_process,
                                                            self.brain,
                                                            self.settings,
                                                            is_api_call=False)

        if self.skip_trigger:
            self.start_order_listener()
        else:
            self.unpause_trigger()

    def on_notification_received(self, notification=None, payload=None):
        logger.debug("[Order] received notification, notification: %s, payload: %s" % (notification, payload))

        # skip_trigger: change the trigger status
        if notification == "skip_trigger":
            self.switch_trigger(payload)

        if notification == "skip_trigger_max_retry":
            self.set_counter_max_retry(payload)

        if notification == "skip_trigger_decrease_max_retry":
            self.decrease_max_retry()

    def set_counter_max_retry(self, payload):
        """
        set a 'counter_max_retry' for max retry before switching automatically the skip_trigger flag to False
        :param payload: payload that contains the max_retry counter value to set
        """
        if "max_retry" in payload:
            if payload["max_retry"] > 0:
                self.counter_max_retry = payload["max_retry"]
                logger.debug("[Order] max_retry set to %s" % self.counter_max_retry)

    def decrease_max_retry(self):
        """
        will decrease the current value of 'counter_max_retry' if the current is > 0
        If the new value == 0, then the skip_trigger flag is automatically switched to False
        """
        logger.debug("[Order] current max_retry: %s" % self.counter_max_retry)
        if self.counter_max_retry > 0:
            self.counter_max_retry = self.counter_max_retry - 1
            logger.debug("[Order] new max_retry value after decrease: %s" % self.counter_max_retry)
            if self.counter_max_retry == 0:
                logger.debug("[Order] max_retry reached '0'. Set skip_trigger to False")
                # the counter raised 0, we can stop to skip the trigger
                self.skip_trigger = False
            
        else:
            logger.debug("[Order] cannot decrease max_retry because current value is <= 0")

    def switch_trigger(self, payload):
        """
        switch the skip_trigger flag
        :param payload: payload dict that contains the new status of the skip_trigger flag
        """
        if "status" in payload:
            status = Utils.str_to_bool(payload["status"])
            if status:
                logger.debug("[Order] switch signals to True")
                self.skip_trigger = True
            else:
                logger.debug("[Order] switch signals to False")
                self.skip_trigger = False



import os
import random
import sndhdr

from kalliope.core.NeuronModule import NeuronModule, MissingParameterException, InvalidParameterException
from kalliope.core.PlayerLauncher import PlayerLauncher


class Play(NeuronModule):
    def __init__(self, **kwargs):
        super(Play, self).__init__(**kwargs)
        self.filename = kwargs.get('filename', None)
        self.player = PlayerLauncher.get_player(settings=self.settings)

        # check if parameters have been provided
        if self._is_parameters_ok():
            self.player.play(self.filename)

    def _is_parameters_ok(self):
        """
        Check if received parameters are ok to perform operations in the neuron
        :return: true if parameters are ok, raise an exception otherwise

        .. raises:: MissingParameterException
        """
        if self.filename is None:
            raise MissingParameterException("You must specify (at least one) filename")
        if not isinstance(self.filename, list):
            self.filename = [self.filename]
        self.filename[:] = [x for x in self.filename if os.path.isfile(x)]
        self.filename[:] = [x for x in self.filename if sndhdr.whathdr(x) is not None]
        self.filename[:] = [x for x in self.filename if sndhdr.whathdr(x).filetype == 'wav']
        if len(self.filename) == 0:
            raise InvalidParameterException("You must specify at least one valid wav file (none remained after validation)")
        self.filename = random.choice(self.filename)
        return True



import os
import inspect
import platform
import shutil
import unittest

from Tests.utils.utils import get_test_path
from kalliope.core.ConfigurationManager import SettingLoader
from kalliope.core.Models import Singleton
from kalliope.core.Models import Resources
from kalliope.core.Models.settings.Options import Options
from kalliope.core.Models.settings.Player import Player
from kalliope.core.Models.settings.RestAPI import RestAPI
from kalliope.core.Models.settings.Settings import Settings
from kalliope.core.Models.settings.Stt import Stt
from kalliope.core.Models.settings.Trigger import Trigger
from kalliope.core.Models.settings.Tts import Tts


class TestSettingLoader(unittest.TestCase):

    def setUp(self):
        self.settings_file_to_test = get_test_path("settings/settings_test.yml")

        self.settings_dict = {
            'rest_api':
                {'allowed_cors_origin': False,
                 'active': True,
                 'login': 'admin',
                 'password_protected': True,
                 'password': 'secret', 'port': 5000},
            'default_trigger': 'snowboy',
            'triggers': [{'snowboy': {'pmdl_file': 'trigger/snowboy/resources/kalliope-FR-6samples.pmdl'}}],
            'default_player': 'mplayer',
            'players': [{'mplayer': {}}, {'pyalsaaudio': {"device": "default"}}],
            'speech_to_text': [{'google': {'language': 'fr-FR'}}],
            'cache_path': '/tmp/kalliope_tts_cache',
            'resource_directory': {
                'stt': '/tmp/kalliope/tests/kalliope_resources_dir/stt',
                'tts': '/tmp/kalliope/tests/kalliope_resources_dir/tts',
                'neuron': '/tmp/kalliope/tests/kalliope_resources_dir/neurons',
                'trigger': '/tmp/kalliope/tests/kalliope_resources_dir/trigger'},
            'default_text_to_speech': 'pico2wave',
            'default_speech_to_text': 'google',
            'text_to_speech': [
                {'pico2wave': {'cache': True, 'language': 'fr-FR'}},
                {'googletts': {'language': 'fr', 'cache': True}}
                ],
            'var_files': ["../Tests/settings/variables.yml"],
            'options': {'energy_threshold': 3000, 'deaf': True, 'mute': False},
            'hooks': {'on_waiting_for_trigger': 'test',
                      'on_stop_listening': None,
                      'on_start_listening': None,
                      'on_order_found': None,
                      'on_start': ['on-start-synapse', 'bring-led-on'],
                      'on_undeaf': [],
                      'on_triggered': ['on-triggered-synapse'],
                      'on_deaf': [],
                      'on_mute': [],
                      'on_unmute': [],
                      'on_order_not_found': [
                          'order-not-found-synapse'],
                      'on_processed_synapses': None,
                      'on_start_speaking': None,
                      'on_stop_speaking': None
                      },
            'send_anonymous_usage_stats': 0
        }

        # Init the folders, otherwise it raises an exceptions
        os.makedirs("/tmp/kalliope/tests/kalliope_resources_dir/neurons")
        os.makedirs("/tmp/kalliope/tests/kalliope_resources_dir/stt")
        os.makedirs("/tmp/kalliope/tests/kalliope_resources_dir/tts")
        os.makedirs("/tmp/kalliope/tests/kalliope_resources_dir/trigger")

    def tearDown(self):
        # Cleanup
        shutil.rmtree('/tmp/kalliope/tests/kalliope_resources_dir')

        Singleton._instances = {}

    def test_singleton(self):
        s1 = SettingLoader(file_path=self.settings_file_to_test)
        s2 = SettingLoader(file_path=self.settings_file_to_test)

        self.assertTrue(s1.settings is s2.settings)

    def test_get_yaml_config(self):

        sl = SettingLoader(file_path=self.settings_file_to_test)
        self.maxDiff = None
        self.assertDictEqual(sl.yaml_config, self.settings_dict)

    def test_get_settings(self):
        settings_object = Settings()
        settings_object.default_tts_name = "pico2wave"
        settings_object.default_stt_name = "google"
        settings_object.default_trigger_name = "snowboy"
        settings_object.default_player_name = "mplayer"
        tts1 = Tts(name="pico2wave", parameters={'cache': True, 'language': 'fr-FR'})
        tts2 = Tts(name="googletts", parameters={'language': 'fr', 'cache': True})
        settings_object.ttss = [tts1, tts2]
        stt = Stt(name="google", parameters={'language': 'fr-FR'})
        settings_object.stts = [stt]
        trigger1 = Trigger(name="snowboy",
                           parameters={'pmdl_file': 'trigger/snowboy/resources/kalliope-FR-6samples.pmdl'})
        settings_object.triggers = [trigger1]
        player1 = Player(name="mplayer", parameters={})
        player2 = Player(name="pyalsaaudio", parameters={"device": "default"})
        settings_object.players = [player1, player2]
        settings_object.rest_api = RestAPI(password_protected=True, active=True,
                                           login="admin", password="secret", port=5000,
                                           allowed_cors_origin=False)
        settings_object.cache_path = '/tmp/kalliope_tts_cache'
        resources = Resources(neuron_folder="/tmp/kalliope/tests/kalliope_resources_dir/neurons",
                              stt_folder="/tmp/kalliope/tests/kalliope_resources_dir/stt",
                              tts_folder="/tmp/kalliope/tests/kalliope_resources_dir/tts",
                              trigger_folder="/tmp/kalliope/tests/kalliope_resources_dir/trigger")
        settings_object.resources = resources
        settings_object.variables = {
            "author": "Lamonf",
            "test_number": 60,
            "test": "kalliope"
        }
        settings_object.options = Options(recognizer_multiplier=1.0, 
                                          recognizer_energy_ratio=1.5,
                                          recognizer_recording_timeout=15.0,
                                          recognizer_recording_timeout_with_silence=3.0,
                                          deaf=True, 
                                          mute=False)
        settings_object.machine = platform.machine()
        settings_object.hooks = {'on_waiting_for_trigger': 'test',
                                 'on_stop_listening': None,
                                 'on_start_listening': None,
                                 'on_order_found': None,
                                 'on_start': ['on-start-synapse', 'bring-led-on'],
                                 'on_undeaf': [],
                                 'on_triggered': ['on-triggered-synapse'],
                                 'on_deaf': [],
                                 'on_mute': [],
                                 'on_unmute': [],
                                 'on_order_not_found': [
                                     'order-not-found-synapse'],
                                 'on_processed_synapses': None,
                                 'on_start_speaking': None,
                                 'on_stop_speaking': None,
                                 }
        settings_object.tracker_anonymous_usage_stats_id = "not_defined_id"

        sl = SettingLoader(file_path=self.settings_file_to_test)

        self.assertEqual(settings_object, sl.settings)

    def test_get_default_speech_to_text(self):
        expected_default_speech_to_text = "google"
        sl = SettingLoader(file_path=self.settings_file_to_test)

        self.assertEqual(expected_default_speech_to_text, sl._get_default_speech_to_text(self.settings_dict))

    def test_get_default_text_to_speech(self):
        expected_default_text_to_speech = "pico2wave"
        sl = SettingLoader(file_path=self.settings_file_to_test)
        self.assertEqual(expected_default_text_to_speech, sl._get_default_text_to_speech(self.settings_dict))

    def test_get_default_trigger(self):
        expected_default_trigger = "snowboy"
        sl = SettingLoader(file_path=self.settings_file_to_test)
        self.assertEqual(expected_default_trigger, sl._get_default_trigger(self.settings_dict))

    def test_get_stts(self):
        stt = Stt(name="google", parameters={'language': 'fr-FR'})
        sl = SettingLoader(file_path=self.settings_file_to_test)
        self.assertEqual([stt], sl._get_stts(self.settings_dict))

    def test_get_ttss(self):
        tts1 = Tts(name="pico2wave", parameters={'cache': True, 'language': 'fr-FR'})
        tts2 = Tts(name="googletts", parameters={'language': 'fr', 'cache': True})
        sl = SettingLoader(file_path=self.settings_file_to_test)
        self.assertEqual([tts1, tts2], sl._get_ttss(self.settings_dict))

    def test_get_triggers(self):
        trigger1 = Trigger(name="snowboy",
                           parameters={'pmdl_file': 'trigger/snowboy/resources/kalliope-FR-6samples.pmdl'})
        sl = SettingLoader(file_path=self.settings_file_to_test)
        self.assertEqual([trigger1], sl._get_triggers(self.settings_dict))

    def test_get_players(self):
        player1 = Player(name="mplayer",
                         parameters={})
        player2 = Player(name="pyalsaaudio",
                         parameters={'device': 'default'})
        sl = SettingLoader(file_path=self.settings_file_to_test)
        self.assertEqual([player1, player2], sl._get_players(self.settings_dict))

    def test_get_rest_api(self):
        expected_rest_api = RestAPI(password_protected=True, active=True,
                                    login="admin", password="secret", port=5000,
                                    allowed_cors_origin=False)

        sl = SettingLoader(file_path=self.settings_file_to_test)
        self.assertEqual(expected_rest_api, sl._get_rest_api(self.settings_dict))

    def test_get_cache_path(self):
        expected_cache_path = '/tmp/kalliope_tts_cache'
        sl = SettingLoader(file_path=self.settings_file_to_test)
        self.assertEqual(expected_cache_path, sl._get_cache_path(self.settings_dict))

    def test_get_resources(self):

        resources = Resources(neuron_folder="/tmp/kalliope/tests/kalliope_resources_dir/neurons",
                              stt_folder="/tmp/kalliope/tests/kalliope_resources_dir/stt",
                              tts_folder="/tmp/kalliope/tests/kalliope_resources_dir/tts",
                              trigger_folder="/tmp/kalliope/tests/kalliope_resources_dir/trigger")
        expected_resource = resources
        sl = SettingLoader(file_path=self.settings_file_to_test)
        self.assertEqual(expected_resource, sl._get_resources(self.settings_dict))

    def test_get_variables(self):
        expected_result = {
            "author": "Lamonf",
            "test_number": 60,
            "test": "kalliope"
        }
        sl = SettingLoader(file_path=self.settings_file_to_test)
        self.assertEqual(expected_result,
                         sl._get_variables(self.settings_dict))

    def test_get_options(self):
        expected_result = Options(recognizer_multiplier=1.0, 
                                  recognizer_energy_ratio=1.5,
                                  recognizer_recording_timeout=15.0,
                                  recognizer_recording_timeout_with_silence=3.0,
                                  deaf=True, 
                                  mute=False)
        sl = SettingLoader(file_path=self.settings_file_to_test)
        self.assertEqual(expected_result,
                         sl._get_options(self.settings_dict))

    def test_get_hooks(self):

        # test with only one hook set
        settings = dict()
        settings["hooks"] = {
            "on_start": "test_synapse"
        }

        expected_dict = {
            "on_start": "test_synapse",
            "on_waiting_for_trigger": None,
            "on_triggered": None,
            "on_start_listening": None,
            "on_stop_listening": None,
            "on_order_found": None,
            "on_order_not_found": None,
            "on_deaf": None,
            "on_undeaf": None,
            "on_mute": None,
            "on_unmute": None,
            "on_start_speaking": None,
            "on_stop_speaking": None
        }

        returned_dict = SettingLoader._get_hooks(settings)

        self.assertEqual(returned_dict, expected_dict)

        # test with no hook set
        settings = dict()

        expected_dict = {
            "on_start": None,
            "on_waiting_for_trigger": None,
            "on_triggered": None,
            "on_start_listening": None,
            "on_stop_listening": None,
            "on_order_found": None,
            "on_order_not_found": None,
            "on_deaf": None,
            "on_undeaf": None,
            "on_mute": None,
            "on_unmute": None,
            "on_start_speaking": None,
            "on_stop_speaking": None
        }

        returned_dict = SettingLoader._get_hooks(settings)

        self.assertEqual(returned_dict, expected_dict)


if __name__ == '__main__':
    unittest.main()

    # suite = unittest.TestSuite()
    # suite.addTest(TestSettingLoader("test_get_hooks"))
    # runner = unittest.TextTestRunner()
    # runner.run(suite)



import collections
import os
import unittest

import mock

from kalliope.core import SignalModule
from kalliope.core.NotificationManager import NotificationManager


class FakeSignal(SignalModule):

    def __init__(self, name=None, **kwargs):
        super(FakeSignal, self).__init__(**kwargs)
        self.name = name

    def on_notification_received(self, notification=None, payload=None):
        pass

    @staticmethod
    def check_parameters(parameters):
        pass


class TestNotificationManager(unittest.TestCase):

    def setUp(self):
        if __name__ == '__main__':
            self.test_path = "__main__.FakeSignal.on_notification_received"
        else:
            current_path = os.getcwd()
            if "Tests" not in current_path:
                self.test_path = "Tests.test_notification_manager.FakeSignal.on_notification_received"
            else:
                self.test_path = "test_notification_manager.FakeSignal.on_notification_received"
        NotificationManager._instances.clear()

    def test_get_instances(self):
        # create a signal
        signal1 = FakeSignal()
        signal2 = FakeSignal()

        expected_list = [
            signal1, signal2
        ]
        compare = lambda x, y: collections.Counter(x) == collections.Counter(y)
        # convert received generator into list
        lst_from_notification = list(NotificationManager.get_instances())
        self.assertTrue(compare(expected_list, lst_from_notification))

    def test_send_notification(self):
        # create a signal
        signal1 = FakeSignal()
        # with mock.patch("__main__.FakeSignal.on_notification_received") \
        with mock.patch(self.test_path) as mock_on_notification_received:
            test_notification = "test"
            NotificationManager.send_notification(test_notification)
            mock_on_notification_received.assert_called_once_with(notification=test_notification, payload=None)
            mock_on_notification_received.reset_mock()


if __name__ == '__main__':
    unittest.main()



import inspect
import logging
import os

from six import with_metaclass

from kalliope.core.ConfigurationManager import SettingLoader
from kalliope.core.ConfigurationManager.ConfigurationChecker import ConfigurationChecker
from kalliope.core.Models import Singleton
from kalliope.core.Models.Brain import Brain
from kalliope.core.Models.Neuron import Neuron
from kalliope.core.Models.Signal import Signal
from kalliope.core.Models.Synapse import Synapse
from kalliope.core.Utils import Utils
from .YAMLLoader import YAMLLoader

logging.basicConfig()
logger = logging.getLogger("kalliope")

FILE_NAME = "brain.yml"


class BrainNotFound(Exception):
    pass


class BrainLoader(with_metaclass(Singleton, object)):
    """
    This Class is used to get the brain YAML and the Brain as an object
    """

    def __init__(self, file_path=None):
        sl = SettingLoader()
        self.settings = sl.settings

        self.file_path = file_path
        if self.file_path is None:  # we don't provide a file path, so search for the default one
            self.file_path = Utils.get_real_file_path(FILE_NAME)
        else:
            self.file_path = Utils.get_real_file_path(file_path)
        # if the returned file path is none, the file doesn't exist
        if self.file_path is None:
            raise BrainNotFound("brain file not found")
        self.yaml_config = self.get_yaml_config()
        self.brain = self.load_brain()

    def get_yaml_config(self):
        """
        Class Methods which loads default or the provided YAML file and return it as a String
        :return: The loaded brain YAML
        :rtype: String

        :Example:
            brain_yaml = BrainLoader.get_yaml_config(/var/tmp/brain.yml)

        .. warnings:: Class Method
        """
        if self.file_path is None:
            brain_file_path = self._get_root_brain_path()
        else:
            brain_file_path = self.file_path
        return YAMLLoader.get_config(brain_file_path)

    def load_brain(self):
        """
        Class Methods which loads default or the provided YAML file and return a Brain
        :return: The loaded Brain
        :rtype: Brain

        :Example:

            brain = BrainLoader.load_brain(file_path="/var/tmp/brain.yml")

        .. seealso:: Brain
        .. warnings:: Class Method
        """

        # Instantiate a brain
        brain = Brain()

        # get the brain with dict
        dict_brain = self.get_yaml_config()

        brain.brain_yaml = dict_brain
        # create list of Synapse
        synapses = list()
        for synapses_dict in dict_brain:
            if "includes" not in synapses_dict:     # we don't need to check includes as it's not a synapse
                if ConfigurationChecker().check_synape_dict(synapses_dict):
                    name = synapses_dict["name"]
                    neurons = self.get_neurons(synapses_dict["neurons"], self.settings)
                    signals = self.get_signals(synapses_dict["signals"])
                    new_synapse = Synapse(name=name, neurons=neurons, signals=signals)
                    synapses.append(new_synapse)
        brain.synapses = synapses
        if self.file_path is None:
            brain.brain_file = self._get_root_brain_path()
        else:
            brain.brain_file = self.file_path
        # check that no synapse have the same name than another
        if not ConfigurationChecker().check_synapes(synapses):
            brain = None

        return brain

    @classmethod
    def get_neurons(cls, neurons_dict, settings):
        """
        Get a list of Neuron object from a neuron dict

        :param neurons_dict: Neuron name or dictionary of Neuron_name/Neuron_parameters
        :type neurons_dict: String or dict
        :param settings:  The Settings with the global variables
        :return: A list of Neurons
        :rtype: List

        :Example:

            neurons = cls._get_neurons(synapses_dict["neurons"])

        .. seealso:: Neuron
        .. warnings:: Static and Private
        """

        neurons = list()
        for neuron_dict in neurons_dict:
            if ConfigurationChecker().check_neuron_dict(neuron_dict):
                if isinstance(neuron_dict, dict):
                    for neuron_name in neuron_dict:
                        new_neuron = Neuron(name=neuron_name, parameters=neuron_dict[neuron_name])
                        neurons.append(new_neuron)
                else:
                    new_neuron = Neuron(name=neuron_dict)
                    neurons.append(new_neuron)
        return neurons

    @classmethod
    def get_signals(cls, signals_dict):
        """
        Get a list of Signal object from a signals dict

        :param signals_dict: Signal name or dictionary of Signal_name/Signal_parameters
        :type signals_dict: String or dict
        :return: A list of Event and/or Order
        :rtype: List

        :Example:

            signals = cls._get_signals(synapses_dict["signals"])

        .. seealso:: Event, Order
        .. warnings:: Class method and Private
        """
        signals = list()
        for signal_dict in signals_dict:
            if ConfigurationChecker().check_signal_dict(signal_dict):
                for signal_name in signal_dict:
                    new_signal = Signal(name=signal_name, parameters=signal_dict[signal_name])
                    signals.append(new_signal)

        return signals

    @staticmethod
    def _get_root_brain_path():
        """
        Return the full path of the default brain file

        :Example:

            brain.brain_file = cls._get_root_brain_path()

        .. raises:: IOError
        .. warnings:: Static method and Private
        """

        # get current script directory path. We are in /an/unknown/path/kalliope/core/ConfigurationManager
        cur_script_directory = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
        # get parent dir. Now we are in /an/unknown/path/kalliope
        parent_dir = os.path.normpath(cur_script_directory + os.sep + os.pardir + os.sep + os.pardir)
        brain_path = parent_dir + os.sep + "brain.yml"
        logger.debug("Real brain.yml path: %s" % brain_path)
        if os.path.isfile(brain_path):
            return brain_path
        raise IOError("Default brain.yml file not found")



import logging

from flask import jsonify, Blueprint
from flask import request

from kalliope import SignalLauncher
from kalliope.core.ConfigurationManager import SettingEditor
from kalliope.core.RestAPI import utils
from kalliope.core.RestAPI.utils import requires_auth

logging.basicConfig()
logger = logging.getLogger("kalliope")

UPLOAD_FOLDER = '/tmp/kalliope/tmp_uploaded_audio'
ALLOWED_EXTENSIONS = {'wav'}


class SettingsView(Blueprint):
    def __init__(self, name, import_name, app=None, brain=None, settings=None):
        self.brain = brain
        self.settings = settings
        self.app = app
        super(SettingsView, self).__init__(name, import_name)
        self.add_url_rule('/settings', view_func=self.get_current_settings, methods=['GET'])
        self.add_url_rule('/settings/deaf/', view_func=self.get_deaf, methods=['GET'])
        self.add_url_rule('/settings/deaf/', view_func=self.set_deaf, methods=['POST'])
        self.add_url_rule('/settings/mute/', view_func=self.get_mute, methods=['GET'])
        self.add_url_rule('/settings/mute/', view_func=self.set_mute, methods=['POST'])
        self.add_url_rule('/settings/recognizer_multiplier/', view_func=self.get_recognizer_multiplier,
                          methods=['GET'])
        self.add_url_rule('/settings/recognizer_multiplier/', view_func=self.set_recognizer_multiplier,
                          methods=['POST'])
        self.add_url_rule('/settings/recognizer_energy_ratio/', view_func=self.get_recognizer_energy_ratio,
                          methods=['GET'])
        self.add_url_rule('/settings/recognizer_energy_ratio/', view_func=self.set_recognizer_energy_ratio,
                          methods=['POST'])
        self.add_url_rule('/settings/recognizer_recording_timeout/', view_func=self.get_recognizer_recording_timeout,
                          methods=['GET'])
        self.add_url_rule('/settings/recognizer_recording_timeout/', view_func=self.set_recognizer_recording_timeout,
                          methods=['POST'])
        self.add_url_rule('/settings/recognizer_recording_timeout_with_silence/', view_func=self.get_recognizer_recording_timeout_with_silence,
                          methods=['GET'])
        self.add_url_rule('/settings/recognizer_recording_timeout_with_silence/', view_func=self.set_recognizer_recording_timeout_with_silence,
                          methods=['POST'])
        self.add_url_rule('/settings/default_tts/', view_func=self.get_default_tts,
                          methods=['GET'])
        self.add_url_rule('/settings/default_tts/', view_func=self.set_default_tts,
                          methods=['POST'])
        self.add_url_rule('/settings/default_stt/', view_func=self.get_default_stt,
                          methods=['GET'])
        self.add_url_rule('/settings/default_stt/', view_func=self.set_default_stt,
                          methods=['POST'])
        self.add_url_rule('/settings/default_player/', view_func=self.get_default_player,
                          methods=['GET'])
        self.add_url_rule('/settings/default_player/', view_func=self.set_default_player,
                          methods=['POST'])
        self.add_url_rule('/settings/default_trigger/', view_func=self.get_default_trigger,
                          methods=['GET'])
        self.add_url_rule('/settings/default_trigger/', view_func=self.set_default_trigger,
                          methods=['POST'])
        self.add_url_rule('/settings/hooks/', view_func=self.get_hooks,
                          methods=['GET'])
        self.add_url_rule('/settings/hooks/', view_func=self.set_hooks,
                          methods=['POST'])
        self.add_url_rule('/settings/variables/', view_func=self.get_variables,
                          methods=['GET'])
        self.add_url_rule('/settings/variables/', view_func=self.set_variables,
                          methods=['POST'])

    @requires_auth
    def get_deaf(self):
        """
        Return the current trigger status

        Curl test
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/deaf
        """

        # find the order signal and call the deaf settings
        if self.settings.options.deaf is not None:
            data = {
                "deaf": self.settings.options.deaf
            }
            return jsonify(data), 200

        # if no Order instance
        data = {
            "error": "deaf status unknow"
        }
        return jsonify(error=data), 400

    @requires_auth
    def set_deaf(self):
        """
        Set the trigger status (deaf or not)

        Curl test:
        curl -i -H "Content-Type: application/json" --user admin:secret  -X POST \
        -d '{"deaf": "True"}' http://127.0.0.1:5000/deaf
        """

        if not request.get_json() or 'deaf' not in request.get_json():
            data = {
                "Error": "Wrong parameters, 'deaf' not set"
            }
            return jsonify(error=data), 400

        # get deaf if present
        deaf = utils.get_value_flag_from_request(http_request=request,
                                                 flag_to_find="deaf",
                                                 is_boolean=True)

        signal_order = SignalLauncher.get_order_instance()
        if signal_order is not None and deaf is not None and self.settings.options.deaf is not None:
            SettingEditor.set_deaf_status(signal_order.trigger_instance, deaf)
            data = {
                "deaf": self.settings.options.deaf
            }
            return jsonify(data), 200

        data = {
            "error": "Cannot switch deaf status"
        }
        return jsonify(error=data), 400

    @requires_auth
    def get_mute(self):
        """
        Return the current mute status

        Curl test
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/mute
        """

        # find the order signal and call the mute settings
        if self.settings.options.mute is not None:
            data = {
                "mute": self.settings.options.mute
            }
            return jsonify(data), 200

        # if no Order instance
        data = {
            "error": "mute status unknow"
        }
        return jsonify(error=data), 400

    @requires_auth
    def set_mute(self):
        """
        Set the Kalliope Core mute status (mute or not)

        Curl test:
        curl -i -H "Content-Type: application/json" --user admin:secret  -X POST \
        -d '{"mute": "True"}' http://127.0.0.1:5000/mute
        """

        if not request.get_json() or 'mute' not in request.get_json():
            data = {
                "Error": "Wrong parameters, 'mute' not set"
            }
            return jsonify(error=data), 400

        # get mute if present
        mute = utils.get_value_flag_from_request(http_request=request,
                                                 flag_to_find="mute",
                                                 is_boolean=True)
        SettingEditor.set_mute_status(mute=mute)

        data = {
            "mute": mute
        }
        return jsonify(data), 200

    @requires_auth
    def get_recognizer_multiplier(self):
        """
        Return the current recognizer_multiplier value from settings

        Curl test
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/settings/recognizer_multiplier
        """

        if self.settings.options.recognizer_multiplier is not None:
            data = {
                "recognizer_multiplier": self.settings.options.recognizer_multiplier
            }
            return jsonify(data), 200

        # if no Order instance
        data = {
            "error": "recognizer_multiplier status not defined"
        }
        return jsonify(error=data), 400

    @requires_auth
    def set_recognizer_multiplier(self):
        """
        Set the Kalliope Core recognizer_multiplier value

        Curl test:
        curl -i -H "Content-Type: application/json" --user admin:secret  -X POST \
        -d '{"recognizer_multiplier": "6666"}' http://127.0.0.1:5000/settings/recognizer_multiplier
        """

        if not request.get_json() or 'recognizer_multiplier' not in request.get_json():
            data = {
                "Error": "Wrong parameters, 'recognizer_multiplier' not set"
            }
            return jsonify(error=data), 400

        # get recognizer_multiplier if present
        recognizer_multiplier = utils.get_value_flag_from_request(http_request=request,
                                                                  flag_to_find="recognizer_multiplier",
                                                                  is_boolean=False)

        SettingEditor.set_recognizer_multiplier(recognizer_multiplier=recognizer_multiplier)

        data = {
            "recognizer_multiplier": recognizer_multiplier
        }
        return jsonify(data), 200

    @requires_auth
    def get_recognizer_energy_ratio(self):
        """
        Return the current recognizer_energy_ratio value from settings

        Curl test
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/settings/recognizer_energy_ratio
        """

        if self.settings.options.recognizer_energy_ratio is not None:
            data = {
                "recognizer_energy_ratio": self.settings.options.recognizer_energy_ratio
            }
            return jsonify(data), 200

        # if no Order instance
        data = {
            "error": "recognizer_energy_ratio status not defined"
        }
        return jsonify(error=data), 400

    @requires_auth
    def set_recognizer_energy_ratio(self):
        """
        Set the Kalliope Core recognizer_energy_ratio value

        Curl test:
        curl -i -H "Content-Type: application/json" --user admin:secret  -X POST \
        -d '{"energy_threshold": "6666"}' http://127.0.0.1:5000/settings/recognizer_energy_ratio
        """

        if not request.get_json() or 'recognizer_energy_ratio' not in request.get_json():
            data = {
                "Error": "Wrong parameters, 'recognizer_energy_ratio' not set"
            }
            return jsonify(error=data), 400

        # get if present
        recognizer_energy_ratio = utils.get_value_flag_from_request(http_request=request,
                                                                    flag_to_find="recognizer_energy_ratio",
                                                                    is_boolean=False)

        SettingEditor.set_recognizer_energy_ratio(recognizer_energy_ratio=recognizer_energy_ratio)

        data = {
            "recognizer_energy_ratio": recognizer_energy_ratio
        }
        return jsonify(data), 200

    @requires_auth
    def get_recognizer_recording_timeout(self):
        """
        Return the current recognizer_recording_timeout value from settings

        Curl test
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/settings/recognizer_recording_timeout
        """

        if self.settings.options.recognizer_recording_timeout is not None:
            data = {
                "recognizer_recording_timeout": self.settings.options.recognizer_recording_timeout
            }
            return jsonify(data), 200

        # if no Order instance
        data = {
            "error": "recognizer_recording_timeout status not defined"
        }
        return jsonify(error=data), 400

    @requires_auth
    def set_recognizer_recording_timeout(self):
        """
        Set the Kalliope Core recognizer_recording_timeout value

        Curl test:
        curl -i -H "Content-Type: application/json" --user admin:secret  -X POST \
        -d '{"energy_threshold": "6666"}' http://127.0.0.1:5000/settings/recognizer_recording_timeout
        """

        if not request.get_json() or 'recognizer_recording_timeout' not in request.get_json():
            data = {
                "Error": "Wrong parameters, 'recognizer_recording_timeout' not set"
            }
            return jsonify(error=data), 400

        # get if present
        recognizer_recording_timeout = utils.get_value_flag_from_request(http_request=request,
                                                                         flag_to_find="recognizer_recording_timeout",
                                                                         is_boolean=False)

        SettingEditor.set_recognizer_recording_timeout(recognizer_recording_timeout=recognizer_recording_timeout)

        data = {
            "recognizer_recording_timeout": recognizer_recording_timeout
        }
        return jsonify(data), 200

    @requires_auth
    def get_recognizer_recording_timeout_with_silence(self):
        """
        Return the current recognizer_recording_timeout_with_silence value from settings

        Curl test
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/settings/recognizer_recording_timeout_with_silence
        """

        if self.settings.options.recognizer_recording_timeout_with_silence is not None:
            data = {
                "recognizer_recording_timeout_with_silence": self.settings.options.recognizer_recording_timeout_with_silence
            }
            return jsonify(data), 200

        # if no Order instance
        data = {
            "error": "recognizer_recording_timeout_with_silence status not defined"
        }
        return jsonify(error=data), 400

    @requires_auth
    def set_recognizer_recording_timeout_with_silence(self):
        """
        Set the Kalliope Core recognizer_recording_timeout_with_silence value

        Curl test:
        curl -i -H "Content-Type: application/json" --user admin:secret  -X POST \
        -d '{"energy_threshold": "6666"}' http://127.0.0.1:5000/settings/recognizer_recording_timeout_with_silence
        """

        if not request.get_json() or 'recognizer_recording_timeout_with_silence' not in request.get_json():
            data = {
                "Error": "Wrong parameters, 'recognizer_recording_timeout_with_silence' not set"
            }
            return jsonify(error=data), 400

        # get if present
        recognizer_recording_timeout_with_silence = utils.get_value_flag_from_request(http_request=request,
                                                                                      flag_to_find="recognizer_recording_timeout_with_silence",
                                                                                      is_boolean=False)

        SettingEditor.set_recognizer_recording_timeout_with_silence(recognizer_recording_timeout_with_silence=recognizer_recording_timeout_with_silence)

        data = {
            "recognizer_recording_timeout_with_silence": recognizer_recording_timeout_with_silence
        }
        return jsonify(data), 200

    @requires_auth
    def get_default_tts(self):
        """
        Return the current default_tts value from settings

        Curl test
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/settings/default_tts
        """

        # the default_tts settings
        if self.settings.default_tts_name is not None:
            data = {
                "default_tts": self.settings.default_tts_name
            }
            return jsonify(data), 200

        # if no Order instance
        data = {
            "error": "default_tts status not defined"
        }
        return jsonify(error=data), 400

    @requires_auth
    def set_default_tts(self):
        """
        Set the Kalliope Core default_tts value

        Curl test:
        curl -i -H "Content-Type: application/json" --user admin:secret  -X POST \
        -d '{"default_tts": "myTTS"}' http://127.0.0.1:5000/settings/default_tts
        """

        if not request.get_json() or 'default_tts' not in request.get_json():
            data = {
                "Error": "Wrong parameters, 'default_tts' not set"
            }
            return jsonify(error=data), 400

        # get if present
        value = utils.get_value_flag_from_request(http_request=request,
                                                  flag_to_find="default_tts",
                                                  is_boolean=False)

        SettingEditor.set_default_tts(default_tts_name=value)

        data = {
            "default_tts": self.settings.default_tts_name
        }
        return jsonify(data), 200

    @requires_auth
    def get_default_stt(self):
        """
        Return the current default_stt value from settings

        Curl test
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/settings/default_stt
        """

        # the default_tts settings
        if self.settings.default_stt_name is not None:
            data = {
                "default_stt": self.settings.default_stt_name
            }
            return jsonify(data), 200

        # if no Order instance
        data = {
            "error": "default_stt status not defined"
        }
        return jsonify(error=data), 400

    @requires_auth
    def set_default_stt(self):
        """
        Set the Kalliope Core default_stt value

        Curl test:
        curl -i -H "Content-Type: application/json" --user admin:secret  -X POST \
        -d '{"default_stt": "myStt"}' http://127.0.0.1:5000/settings/default_stt
        """

        if not request.get_json() or 'default_stt' not in request.get_json():
            data = {
                "Error": "Wrong parameters, 'default_stt' not set"
            }
            return jsonify(error=data), 400

        # get if present
        value = utils.get_value_flag_from_request(http_request=request,
                                                  flag_to_find="default_stt",
                                                  is_boolean=False)

        SettingEditor.set_default_stt(default_stt_name=value)

        data = {
            "default_stt": self.settings.default_stt_name
        }
        return jsonify(data), 200

    @requires_auth
    def get_default_player(self):
        """
        Return the current default_player value from settings

        Curl test
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/settings/default_player
        """

        # the default_player settings
        if self.settings.default_player_name is not None:
            data = {
                "default_player": self.settings.default_player_name
            }
            return jsonify(data), 200

        # if no Order instance
        data = {
            "error": "default_player status not defined"
        }
        return jsonify(error=data), 400

    @requires_auth
    def set_default_player(self):
        """
        Set the Kalliope Core default_player value

        Curl test:
        curl -i -H "Content-Type: application/json" --user admin:secret  -X POST \
        -d '{"default_player": "myPlayer"}' http://127.0.0.1:5000/settings/default_player
        """

        if not request.get_json() or 'default_player' not in request.get_json():
            data = {
                "Error": "Wrong parameters, 'default_player' not set"
            }
            return jsonify(error=data), 400
        # get if present
        value = utils.get_value_flag_from_request(http_request=request,
                                                  flag_to_find="default_player",
                                                  is_boolean=False)

        SettingEditor.set_default_player(default_player_name=value)

        data = {
            "default_player": self.settings.default_player_name
        }
        return jsonify(data), 200

    @requires_auth
    def get_default_trigger(self):
        """
        Return the current default_trigger value from settings

        Curl test
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/settings/default_trigger
        """

        # the default_trigger settings
        if self.settings.default_trigger_name is not None:
            data = {
                "default_trigger": self.settings.default_trigger_name
            }
            return jsonify(data), 200

        # if no Order instance
        data = {
            "error": "default_trigger status not defined"
        }
        return jsonify(error=data), 400

    @requires_auth
    def set_default_trigger(self):
        """
        Set the Kalliope Core default_trigger value

        Curl test:
        curl -i -H "Content-Type: application/json" --user admin:secret  -X POST \
        -d '{"default_trigger": "myTrigger"}' http://127.0.0.1:5000/settings/default_trigger
        """

        if not request.get_json() or 'default_trigger' not in request.get_json():
            data = {
                "Error": "Wrong parameters, 'default_trigger' not set"
            }
            return jsonify(error=data), 400

        # get if present
        value = utils.get_value_flag_from_request(http_request=request,
                                                  flag_to_find="default_trigger",
                                                  is_boolean=False)

        SettingEditor.set_default_trigger(default_trigger=value)

        data = {
            "default_trigger": self.settings.default_trigger_name
        }
        return jsonify(data), 200

    @requires_auth
    def get_hooks(self):
        """
        Return the list of hooks from settings

        Curl test
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/settings/hooks
        """

        if self.settings.hooks is not None:
            data = {
                "hooks": self.settings.hooks
            }
            return jsonify(data), 200

        # if no Order instance
        data = {
            "error": "hooks are not defined"
        }
        return jsonify(error=data), 400

    @requires_auth
    def set_hooks(self):
        """
        Set the Kalliope Core hooks value

        Curl test:
        curl -i -H "Content-Type: application/json" --user admin:secret  -X POST \
        -d '{"hook_name":"hook_value","hooke_name2":"hook_value2"}' http://127.0.0.1:5000/settings/hooks
        """

        if not request.get_json():
            data = {
                "Error": "No Parameters not"
            }
            return jsonify(error=data), 400

        # get if present
        value = request.get_json()

        if not isinstance(value, dict):
            data = {
                "Error": "Hooks must be a dictionary"
            }
            return jsonify(error=data), 400

        SettingEditor.set_hooks(hooks=value)

        data = {
            "hooks": self.settings.hooks
        }
        return jsonify(data), 200

    @requires_auth
    def get_variables(self):
        """
        Return the list of variables from settings

        Curl test
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/settings/variables
        """

        if self.settings.variables is not None:
            data = {
                "variables": self.settings.variables
            }
            return jsonify(data), 200

        # if no Order instance
        data = {
            "error": "variables are not defined"
        }
        return jsonify(error=data), 400

    @requires_auth
    def set_variables(self):
        """
        Set the Kalliope Core variables value.
        Can be used with a dictionary of variables :
        curl -i -H "Content-Type: application/json" --user admin:secret  -X POST \
        -d '{"variable1":"variables_value","variables_name2":"variables_value2"}' http://127.0.0.1:5000/settings/varaibles
        """

        if not request.get_json():
            data = {
                "Error": "No Parameters provided"
            }
            return jsonify(error=data), 400

        # get if present
        value = request.get_json()

        if not isinstance(value, dict):
            data = {
                "Error": "Variables must be a dictionary"
            }
            return jsonify(error=data), 400

        SettingEditor.set_variables(variables=value)

        data = {
            "variables": self.settings.variables
        }
        return jsonify(data), 200

    def get_current_settings(self):
        """
        get the current settings config
        test with curl:
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/settings
        """
        logger.debug("[FlaskAPI] get_current_settings: all")
        data = jsonify(settings=self.settings.serialize())
        return data, 200



import logging
import os
import time

from flask import jsonify, Blueprint
from flask import request
from werkzeug.utils import secure_filename

from kalliope import Utils
from kalliope.core.ConfigurationManager import BrainLoader, SettingEditor
from kalliope.core.ConfigurationManager.ConfigurationChecker import KalliopeModuleNotFoundError, ConfigurationChecker, \
    InvalidSynapeName, NoSynapeNeurons, NoSynapeSignals
from kalliope.core.Cortex import Cortex
from kalliope.core.Lifo.LifoManager import LifoManager
from kalliope.core.Models import Synapse
from kalliope.core.Models.MatchedSynapse import MatchedSynapse
from kalliope.core.OrderListener import OrderListener
from kalliope.core.RestAPI import utils
from kalliope.core.RestAPI.utils import requires_auth
from kalliope.core.SynapseLauncher import SynapseLauncher

logging.basicConfig()
logger = logging.getLogger("kalliope")

UPLOAD_FOLDER = '/tmp/kalliope/tmp_uploaded_audio'
ALLOWED_EXTENSIONS = {'wav'}


class SynapsesView(Blueprint):
    def __init__(self, name, import_name, app, brain=None, settings=None):
        self.brain = brain
        self.settings = settings
        self.app = app
        super(SynapsesView, self).__init__(name, import_name)

        # api_response sent by the Order Analyser when using the /synapses/start/audio URL
        self.api_response = None
        # boolean used to notify the main process that we get the list of returned synapse
        self.order_analyser_return = False

        # routes
        self.add_url_rule('/synapses', view_func=self.get_synapses, methods=['GET'])
        self.add_url_rule('/synapses', view_func=self.create_synapses, methods=['POST'])
        self.add_url_rule('/synapses/<synapse_name>', view_func=self.get_synapse, methods=['GET'])
        self.add_url_rule('/synapses/<synapse_name>', view_func=self.delete_synapse, methods=['DELETE'])
        self.add_url_rule('/synapses/start/id/<synapse_name>', view_func=self.run_synapse_by_name, methods=['POST'])
        self.add_url_rule('/synapses/start/order', view_func=self.run_synapse_by_order, methods=['POST'])
        self.add_url_rule('/synapses/start/audio', view_func=self.run_synapse_by_audio, methods=['POST'])

    def _get_synapse_by_name(self, synapse_name):
        """
        Find a synapse in the brain by its name
        :param synapse_name:
        :return:
        """
        all_synapse = self.brain.synapses
        for synapse in all_synapse:
            try:
                if synapse.name == synapse_name:
                    return synapse
            except KeyError:
                pass
        return None

    @requires_auth
    def create_synapses(self):
        """
        curl -i -H "Content-Type: application/json" \
        --user admin:secret \
        -X POST \
        -d '{
          "name": "Say-hello",
          "signals": [
            {
              "order": "je suis nicolas"
            }
          ],
          "neurons": [
            {
              "say": {
                "message": "je sais"
              }
            }
          ]
        }' \
        http://127.0.0.1:5000/synapses
        :return:
        """
        if not request.get_json() or 'name' not in request.get_json():
            data = {
                "Error": "Wrong parameters, 'name' not set"
            }
            return jsonify(error=data), 400

        new_synapse = request.get_json()
        try:
            ConfigurationChecker().check_synape_dict(new_synapse)
        except (InvalidSynapeName, NoSynapeNeurons, NoSynapeSignals) as e:
            data = {
                "error": "%s" % e
            }
            return jsonify(data), 400

        try:
            name = new_synapse["name"]
            neurons = BrainLoader.get_neurons(new_synapse["neurons"], self.settings)
            signals = BrainLoader.get_signals(new_synapse["signals"])
            new_synapse_instance = Synapse(name=name, neurons=neurons, signals=signals)
            self.brain.synapses.append(new_synapse_instance)
            # TODO save the brain in yaml
            return jsonify(new_synapse_instance.serialize()), 201
        except KalliopeModuleNotFoundError as e:
            data = {
                "error": "%s" % e
            }
            return jsonify(data), 400

    @requires_auth
    def get_synapses(self):
        """
        get all synapses.
        test with curl:
        curl -i --user admin:secret  -X GET  http://127.0.0.1:5000/synapses
        """
        logger.debug("[FlaskAPI] get_synapses: all")
        data = jsonify(synapses=[e.serialize() for e in self.brain.synapses])
        return data, 200

    @requires_auth
    def get_synapse(self, synapse_name):
        """
        get a synapse by its name
        test with curl:
        curl --user admin:secret -i -X GET  http://127.0.0.1:5000/synapses/say-hello-en
        """
        logger.debug("[FlaskAPI] get_synapse: synapse_name -> %s" % synapse_name)
        synapse_target = self._get_synapse_by_name(synapse_name)
        if synapse_target is not None:
            data = jsonify(synapses=synapse_target.serialize())
            return data, 200

        data = {
            "synapse name not found": "%s" % synapse_name
        }
        return jsonify(error=data), 404

    @requires_auth
    def delete_synapse(self, synapse_name):
        """
        delete a synapse by its name
        test with curl:
        curl --user admin:secret -i -X DELETE  http://127.0.0.1:5000/synapses/say-hello-en
        """
        logger.debug("[FlaskAPI] delete_synapse -> %s" % synapse_name)
        synapse_target = self._get_synapse_by_name(synapse_name)
        if synapse_target is not None:
            # delete from brain
            self._delete_synapse_by_name(synapse_name)
            return '', 204

        data = {
            "synapse name not found": "%s" % synapse_name
        }
        return jsonify(error=data), 404

    @requires_auth
    def run_synapse_by_name(self, synapse_name):
        """
        Run a synapse by its name
        test with curl:
        curl -i --user admin:secret -X POST  http://127.0.0.1:5000/synapses/start/id/say-hello-fr

        run a synapse without making kalliope speaking
        curl -i -H "Content-Type: application/json" --user admin:secret -X POST  \
        -d '{"mute":"true"}' http://127.0.0.1:5000/synapses/start/id/say-hello-fr

        Run a synapse by its name and pass order's parameters
        curl -i -H "Content-Type: application/json" --user admin:secret -X POST  \
        -d '{"mute":"true", "parameters": {"parameter1": "value1" }}' \
        http://127.0.0.1:5000/synapses/start/id/say-hello-fr

        :param synapse_name: name(id) of the synapse to execute
        :return:
        """
        # get a synapse object from the name
        logger.debug("[FlaskAPI] run_synapse_by_name: synapse name -> %s" % synapse_name)
        synapse_target = BrainLoader().brain.get_synapse_by_name(synapse_name=synapse_name)

        # Store the mute value, then apply depending of the request parameters
        old_mute_value = self.settings.options.mute
        mute = utils.get_value_flag_from_request(http_request=request,
                                                 flag_to_find="mute",
                                                 is_boolean=True)
        if mute is not None:
            SettingEditor.set_mute_status(mute=mute)

        # get parameters
        parameters = utils.get_parameters_from_request(request)

        if synapse_target is None:
            data = {
                "synapse name not found": "%s" % synapse_name
            }
            if mute is not None:
                SettingEditor.set_mute_status(mute=old_mute_value)
            return jsonify(error=data), 404
        else:
            # generate a MatchedSynapse from the synapse
            matched_synapse = MatchedSynapse(matched_synapse=synapse_target, overriding_parameter=parameters)
            # get the current LIFO buffer from the singleton
            lifo_buffer = LifoManager.get_singleton_lifo()
            lifo_buffer.add_synapse_list_to_lifo([matched_synapse])
            response = lifo_buffer.execute(is_api_call=True)
            data = jsonify(response)
            if mute is not None:
                SettingEditor.set_mute_status(mute=old_mute_value)
            return data, 201

    @requires_auth
    def run_synapse_by_order(self):
        """
        Give an order to Kalliope via API like it was from a spoken one
        Test with curl
        curl -i --user admin:secret -H "Content-Type: application/json" -X POST \
        -d '{"order":"my order"}' http://localhost:5000/synapses/start/order

        In case of quotes in the order or accents, use a file
        cat post.json:
        {"order":"j'aime"}
        curl -i --user admin:secret -H "Content-Type: application/json" -X POST \
        --data @post.json http://localhost:5000/order/

        Can be used with mute flag
        curl -i --user admin:secret -H "Content-Type: application/json" -X POST \
        -d '{"order":"my order", "mute":"true"}' http://localhost:5000/synapses/start/order

        :return:
        """
        if not request.get_json() or 'order' not in request.get_json():
            data = {
                "Error": "Wrong parameters, 'order' not set"
            }
            return jsonify(error=data), 400

        order = request.get_json('order')

        # Store the mute value, then apply depending of the request parameters
        old_mute_value = self.settings.options.mute
        mute = utils.get_value_flag_from_request(http_request=request,
                                                 flag_to_find="mute",
                                                 is_boolean=True)
        if mute is not None:
            SettingEditor.set_mute_status(mute=mute)

        if order is not None:
            # get the order
            order_to_run = order["order"]
            logger.debug("[FlaskAPI] run_synapse_by_order: order to run -> %s" % order_to_run)
            api_response = SynapseLauncher.run_matching_synapse_from_order(order_to_run,
                                                                           self.brain,
                                                                           self.settings,
                                                                           is_api_call=True)

            Cortex.save('kalliope_last_order', order_to_run)
            data = jsonify(api_response)
            if mute is not None:
                SettingEditor.set_mute_status(mute=old_mute_value)
            return data, 201
        else:
            data = {
                "error": "order cannot be null"
            }
            if mute is not None:
                SettingEditor.set_mute_status(mute=old_mute_value)
            return jsonify(error=data), 400

    @requires_auth
    def run_synapse_by_audio(self):
        """
        Give an order to Kalliope with an audio file
        Test with curl
        curl -i --user admin:secret -X POST  http://localhost:5000/synapses/start/audio -F "file=@/path/to/input.wav"

        With mute flag
        curl -i --user admin:secret -X POST \
        http://localhost:5000/synapses/start/audio -F "file=@path/to/file.wav" -F mute="true"
        :return:
        """

        # check if the post request has the file part
        if 'file' not in request.files:
            data = {
                "error": "No file provided"
            }
            return jsonify(error=data), 400

        uploaded_file = request.files['file']
        # if user does not select file, browser also
        # submit a empty part without filename
        if uploaded_file.filename == '':
            data = {
                "error": "No file provided"
            }
            return jsonify(error=data), 400

        # Store the mute value, then apply depending of the request parameters
        old_mute_value = self.settings.options.mute
        if request.form.get("mute"):
            SettingEditor.set_mute_status(mute=Utils.str_to_bool(request.form.get("mute")))

        # save the file
        filename = secure_filename(uploaded_file.filename)
        base_path = os.path.join(self.app.config['UPLOAD_FOLDER'])
        uploaded_file.save(os.path.join(base_path, filename))

        # now start analyse the audio with STT engine
        audio_path = base_path + os.sep + filename
        logger.debug("[FlaskAPI] run_synapse_by_audio: with file path %s" % audio_path)
        if not self.allowed_file(audio_path):
            audio_path = self._convert_to_wav(audio_file_path=audio_path)
        ol = OrderListener(callback=self.audio_analyser_callback, audio_file_path=audio_path)
        ol.start()
        ol.join()
        # wait the Order Analyser processing. We need to wait in this thread to keep the context
        while not self.order_analyser_return:
            time.sleep(0.1)
        self.order_analyser_return = False
        if self.api_response is not None and self.api_response:
            data = jsonify(self.api_response)
            self.api_response = None
            logger.debug("[FlaskAPI] run_synapse_by_audio: data %s" % data)
            if request.form.get("mute"):
                SettingEditor.set_mute_status(mute=old_mute_value)
            return data, 201
        else:
            data = {
                "error": "The given order doesn't match any synapses"
            }
            if request.form.get("mute"):
                SettingEditor.set_mute_status(mute=old_mute_value)
            return jsonify(error=data), 400

    def audio_analyser_callback(self, order):
        """
        Callback of the OrderListener. Called after the processing of the audio file
        This method will
        - call the Order Analyser to analyse the  order and launch corresponding synapse as usual.
        - get a list of launched synapse.
        - give the list to the main process via self.launched_synapses
        - notify that the processing is over via order_analyser_return
        :param order: string order to analyse
        :return:
        """
        logger.debug("[FlaskAPI] audio_analyser_callback: order to process -> %s" % order)
        api_response = SynapseLauncher.run_matching_synapse_from_order(order,
                                                                       self.brain,
                                                                       self.settings,
                                                                       is_api_call=True)
        self.api_response = api_response
        Cortex.save('kalliope_last_order', order)
        # this boolean will notify the main process that the order have been processed
        self.order_analyser_return = True

    @staticmethod
    def _convert_to_wav(audio_file_path):
        """
        If not already .wav, convert an incoming audio file to wav format. Using system avconv (raspberry)
        :param audio_file_path: the current full file path
        :return: Wave file path
        """
        # Not allowed so convert into wav using avconv (raspberry)
        base = os.path.splitext(audio_file_path)[0]
        extension = os.path.splitext(audio_file_path)[1]
        if extension != ".wav":
            current_file_path = audio_file_path
            logger.debug("Converting file " + current_file_path + " to .wav")
            audio_file_path = base + ".wav"
            os.system("ffmpeg -loglevel panic -y -i " + current_file_path + " " + audio_file_path)  # --> deprecated
            # subprocess.call(['avconv', '-y', '-i', audio_path, new_file_path], shell=True) # Not working ...

        return audio_file_path

    @staticmethod
    def allowed_file(filename):
        return '.' in filename and \
               filename.rsplit('.', 1)[1].lower() in ALLOWED_EXTENSIONS

    def _delete_synapse_by_name(self, synapse_name):
        all_synapse = self.brain.synapses
        for synapse in all_synapse:
            try:
                if synapse.name == synapse_name:
                    logger.debug("[FlaskAPI] remove synapse from the brain: '%s'" % synapse_name)
                    all_synapse.remove(synapse)
                    # TODO save the brain in yaml
            except KeyError:
                pass
        return None
